#include <avr/io.h>
#include <util/delay.h>

#define setOutput(directions, pin) (directions |= (1 << pin))
#define digitalWriteHigh(port, pin) (port |= (1 << pin))
#define digitalWriteLow(port, pin) (port &= ~(1 << pin))
#define time_delay() _delay_ms(500)

int main(void) {

    /* Using macros defined above: */ 
    setOutput(DDRB, DDB5); 
    /* Bit masking: 
     * (1 << DDxn) means setting OUTPUT at DDRxn level  
     * (0 << DDxn) means setting INPUT at DDRxn level  
     * _BV(DDB5) equals (1 << DDB5)
     */
    // DDRB |= _BV(DDB5);  
    // or DDRB = (1 << DDB5) | DDRB; 
    // or DDRB |= (1 << DDB5); 
    /* Plain AVR C: 
     * the quantity of numbers after 0b are the numbers of PBn pins   
     * since we have 6 PBn pins with PB0 as the first pin and PB5 as the last pin 
     * then PB5 will be the first number after 0b and PB0 as the last number 
     * in this case 0b100000 means setting PB5 as OUTPUT
     */ 
    // DDRB = 0b100000; 

    while (1) {
      
      /* Using macros defined above: */  
      digitalWriteHigh(PORTB, PORTB5); 
      /* Bit masking: 
       * (1 << PORTxn) means setting HIGH at PINxn level  
       * | OR operation
       */ 
      // PORTB |= _BV(PORTB5);
      // or PORTB = (1 << PORTB5) | PORTB; 
      // or PORTB |= (1 << PORTB5); 
      /* Plain AVR C: 
       * 0b100000 means setting PB5 as HIGH
       */ 
      // PORTB = 0b100000; 

      time_delay();

      /* Using macros defined above: */ 
      digitalWriteLow(PORTB, PORTB5); 
      /* Bit masking: 
       * (0 << PORTxn) means setting LOW at PINxn level  
       * & AND operation
       * ~ NOT operation
       */  
      // PORTB &= ~_BV(PORTB5); 
      // or PORTB = ~ (1 << PORTB5) & PORTB; 
      // or PORTB = (0 << PORTB5) | PORTB; 
      /* Plain AVR C: 
       * 0b000000 means setting PB5 as LOW
       */  
      // PORTB = 0b000000; 
         
      time_delay();
    }
}
