---
layout: assignment
title: Embedded Programming
thumb: 09.jpg
assignment: read a micro-controller data sheet (individual); to program the designed board (individual); to compare different architectures (in-group). 
active: 1
---
<p class="font-italic font-weight-bold">* All source and config files can be downloaded <a href="https://gitlab.fabcloud.org/academany/fabacademy/2020/labs/barcelona/students/tue-ngo/tree/master/assets/img/ccpp">here</a>.</p>

<p class="font-italic font-weight-bold">* To keep the momentum of the 4th and 6th weeks, I tried to write down what I understand about the <a href="#theory">embedded development process</a>.</p>

<p class="font-italic font-weight-bold">* I need to individually read a <a href="#datasheet">micro-controller data sheet</a> and program the board previously designed in <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-06-electronics-design.html#dice">the 6th week</a>.</p>

<p class="font-italic font-weight-bold">* This week's group assignment is to compare the performance and development workflows of <a href="#group">different micro-controller families.</a></p>

<h2 id="theory">Understanding the embedded systems and programming process</h2>
<p>I personally find the theory sections I wrote down during electronics weeks to be really useful since they helped me organizing completely new pieces of knowledge. This week, guest instructor Guillem Campodron gave us a very <a href="https://speakerdeck.com/pral2a/how-to-make-a-computer">informative session</a> which untangled many concepts I didn't understand about embedded systems. Embedded systems are self-contained control systems or computer systems designed for a particular purpose with bare necessary peripherals needed to run it. The core part of the embedded system is the processor (could be a micro-controller, an <a href="https://en.wikipedia.org/wiki/Field-programmable_gate_array">FPGA</a>, or an <a href="https://en.wikipedia.org/wiki/System_on_a_chip">SoC</a>). Different interfaces (parallel or serial communication interfaces, Wifi, USB, etc.) and peripherals (displays, SD cards, input and output devices, etc.) are interfaced to the system.</p>
<p>I also took a look at this <a href="https://books.google.es/books?id=y7C2AgAAQBAJ&pg=PT36&lpg=PT36&dq=compiling+flashing+hardware&source=bl&ots=LAKkBt3JY-&sig=ACfU3U3GpUDEj9n-xuqbNFnVjXIWX4677w&hl=en&sa=X&ved=2ahUKEwjo5IPjqLnoAhVQ3IUKHQQ5CkgQ6AEwAXoECAoQAQ#v=onepage&q=compiling%20flashing%20hardware&f=false">book</a> to get to know better how to program my tinyAVR board. In short, in order to write software to target hardware, we need to go through a process of coding, compiling and flashing the programs into the chips. Allow me to go through a very long note without any images attached, or go straight to the assignments <a href="#assignment">below.</a></p>
<h5>Target hardware - Micro-controller unit (MCU)</h5>
<p>A typical <a href="https://en.wikipedia.org/wiki/Microcontroller">micro-controller</a> includes a <strong>core</strong> (a processor, memory, clocks, and input/output pins) and <strong>peripherals</strong> on a single chip. The micro-controller controls a singular function by interpreting data received from I/O peripherals using the central processor. The temporary information that the micro-controller receives is stored in its data memory, where the processor accesses it and uses instructions (firmware) stored in its program memory to decipher and apply the incoming data. It then uses its on-chip I/O peripherals to communicate and enact the appropriate action. There are many available micro-controller families, some of them are widely used such as AVR (tinyAVR and megaAVR), ARM (ARM-x, Cortex-A/R/M), or Xtensa (ESP32, ESP8266), etc. For example, the ATTiny1614 chip used in my LED dice board is from tinyAVR family. Due to the COVID-19 outbreak situation, we were also provided by the instructors an <a href="https://en.wikipedia.org/wiki/Arduino_Uno">Arduino Uno</a> board with ATmega328P chip, <a href="https://www.luisllamas.es/esp8266-nodemcu/">NodeMCU</a> board with ESP8266MOD chip and a <a href="https://gitlab.fabcloud.org/barcelonaworkshops/barduino-2.0">Barduino</a> board with ESP32-WROOM chip.</p>
<p></p>
<h6>The core: Processor, Memory and I/O</h6>
<ul>
<li>Central processor (similar to CPU on a computer): ranging from small and simple 4-bit processors to complex 64-bit processors. For example, my ATtiny1614 micro-controllers uses the 8-bit AVR processor. A processor can be considered as the brain of the device. It can respond to events and perform basic logic and I/O operations. It also performs data transfer operations, which communicate commands to other components in the larger embedded system.</li>
<li>Memory: volatile memory (RAM) for temporary data storage and read-only ROM, EPROM, EEPROM or Flash memory for program (instructions) storage.</li>
<li>Clocks: micro-controllers need a clock signal to trigger their command signals/instructions according to the conditions/timings set by the program.</li>
<li>Inputs and outputs (GPIO controllers).</li>
</ul>
<p></p>
<h6>I/O peripherals (on-chip)</h6>
<p>Peripherals are devices that aid the micro-processor to accomplish a given job. Depending on their location, they can be classified into 2 types. If they are located inside the micro-controller chip, they are called on-chip peripherals. If they are located outside the chip but on the same PCB, they are called off-chip peripherals. Some examples of on-chip peripherals:</p>
<ul>
<li>Analog-to-digital converters (ADC) or digital-to-analog converters (DAC): An ADC converts analog signals to digital signals and allows the processor to interface with external analog devices, such as sensors. A DAC allows the processor to communicate its outgoing signals to external analog components.</li>
<li id="serial">Serial communication controllers: allow the micro-controller to communicate with external components. They have a similar function to a USB or a parallel port but differ in the way they exchange bits: I2C, SPI, UART. In short, UART is useful for communicating with computers, radio modems, and GPS units. SPI is good for ultra-fast communication over very short distances with peripherals like memories, ADCs, and DACs. I2C is a good choice for devices that move around a moderate amount of data and connect to multiple sensors and motors. More information can be found <a href="https://maker.pro/arduino/tutorial/common-communication-peripherals-on-the-arduino-uart-i2c-and-spi">here.</a></li>
<li>In-circuit programming and in-circuit debugging support.</li>
<li>Interrupt controllers: listen to the peripherals for events and report to the processor once events occur. The particular event that we need to listen to can be programmed and unnecessary events can be ignored. Once a registered event happens, the interrupt controller goes and tells the processor about it, so that the processor can take appropriate actions.</li>
</ul>
<p></p>

<h5>Coding/Compiling/Linking/Debugging - Toolchain</h5>
<p>A <a href="https://elinux.org/Toolchains">toolchain</a> is a set of software development tools that are chained together as a cycle of specific stages such as code, compile, run and debug. <a href="https://en.wikipedia.org/wiki/GNU_toolchain">GNU toolchain</a> which consists of a C compiler is widely used in programming embedded systems and most toolchains are developed base on GNU (for instance, <a href="https://www.microchip.com/mplab/avr-support/avr-and-arm-toolchains-c-compilers">AVR GNU toolchain and ARM GNU toolchain</a>). A toolchain usually consists of:</p>
<ul>
<li>Editor: editing source code to control embedded systems. It could be a standalone application or built into an IDE/IDP.</li>
<li>Compiler: transforming the code into object code written by low-level language that a machine can understand. <a href="https://gcc.gnu.org/">GCC</a> is the compiler included in GNU toolchain. <a href="https://en.wikipedia.org/wiki/Make_(software)">Make</a> is an automation tool for compiling and building programs using avr-gcc compiler and avrdude programmer, hence it can transfer the <code>.hex</code> file into the target chip.</li> 
<li>Assembler: converting <a href="https://en.wikipedia.org/wiki/Assembly_language">assembly code</a> into executable machine code. It takes the basic commands and operations from assembly code and converts them into <a href="https://en.wikipedia.org/wiki/Intel_HEX">hex</a>/binary code that can be recognized by a specific type of processor. IDEs often include assemblers.</li> 
<li>Linker: combining all small pieces and modules of code together, creating an executable program. The linker of GNU toolchain is called <a href="https://ftp.gnu.org/old-gnu/Manuals/ld-2.9.1/html_mono/ld.html">ld</a>.</li>
<li>Debugger: testing and debugging programs. There are both source-level debugger or machine-language debugger. <a href="https://en.wikipedia.org/wiki/GNU_Debugger">GDB</a> is the GNU debugger.</li>
<li>Integrated development editor/platform: An IDE/IDP often consists of a source code editor, a compiler, a linker, and a debugger. <a href="https://www.arduino.cc/en/main/software">Arduino IDE</a> and <a href="https://www.microchip.com/mplab/avr-support/atmel-studio-7">Atmel Studio</a> are widely used by electronics lovers, and we were also introduced to <a href="https://docs.platformio.org/en/latest/what-is-platformio.html">PlatformIO</a> by our instructor Oscar.</li>
<li>Libraries: collections of code, such as an API, that allow the app to reference prebuilt functions or other resources. <a href="https://en.wikipedia.org/wiki/GNU_C_Library">glibc</a> provides the core libraries for the GNU system as well as many other systems that use Linux as the <a href="https://en.wikipedia.org/wiki/Kernel_(operating_system)">kernel</a>.</li>
<li>Frameworks: an abstraction layer in which software providing generic functionality can be selectively changed by additional user-written code. Some examples: <a href="https://www.arduino.cc/reference/en/">Arduino</a> framework based on <a href="http://wiring.org.co/">Wiring</a> (AVR, ARM), <a href="https://www2.keil.com/mdk5/cmsis">CMSIS</a> framework (Cortex-M/A), <a href="https://github.com/espressif/esp-idf?utm_source=platformio&utm_medium=docs">ESP-IDF</a> framework (ESP32), etc.</li>
</ul>
<p>In summary, a toolchain is able to compile source code written in a high-level language into executables that can run on target devices. Most of the programs are written in C/C++, but there are also interpreters such as <a href="https://en.wikipedia.org/wiki/Espruino">Espruino</a> that supports writing code in JavaScript and <a href="https://micropython.org/">MicroPython</a> that supports writing code in Python 3. Both of them also have their own compatible hardware and software as well as unique features.</p>
<p></p>

<h5>Flashing - In-system programmer</h5>
<p>To flash a program to a chip, we might need extra hardware (an ISP) and software (the program that will communicate with the hardware programmer). I wrote about ISPs before, in the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-04-electronics-production.html#isp">4th week</a>. Basically, an external ISP is an extra bit of hardware that connects between a host computer and the target board and allows the microcontroller to be programmed. Besides, depends on the chip, sometimes we have bootloader which is a piece of firmware stored in ROM that allows installing new firmware without the need of an external programmer.</p>
<p>In many cases, we need programming utilities to download and upload the on-chip memories. <a href="https://www.nongnu.org/avrdude/">avrdude</a> is a flashing utility working with <a href="https://gcc.gnu.org/wiki/avr-gcc">avr-gcc</a> compiler/linker and can be used effectively via the command line or within Arduino IDE. Other examples are pyupdi (AVR chips - UPDI interface), edbg (ARM chips - CMSIS interface) or esptool.py (Espressif chips), etc.</p>
<p></p>

<h2 id="assignment">Understanding electronics data sheets</h2>
<h5 id="datasheet">Individual assignment - Read the ATtiny1614 data sheet</h5>
<p>I indeed took a look at this <a href="https://github.com/SpenceKonde/megaTinyCore/blob/master/megaavr/extras/ATtiny_x14.md">pinout diagram</a> and this <a href="http://ww1.microchip.com/downloads/en/DeviceDoc/ATtiny1614-data-sheet-40001995A.pdf">data sheet</a> <strong>BEFORE</strong> designing my board in the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-06-electronics-design.html">6th week</a> in order to know where should I connect the FTDI and UPDI headers to the chip. The RXD pin of the chip will be connected to the TX pin of the FTDI header, and the TXD pin of the chip will be connected to the RX pin of the FTDI header. This is the basic of UART serial communication.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/datasheet-2.png" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/datasheet-1.png" class="img-fluid w-100"/>
<p><strong>Here are some specs of the ATtiny1614 chip:</strong></p>
<ul>
<li>Flash (program memory): 16384 bytes</li>
<li>EEPROM: 256 bytes</li>
<li>RAM: 2048 bytes</li>
<li>General-purpose I/O Pins: 12 (11 usable)</li>
<li>Digital pins: 2</li>
<li>ADC pins: 9 usable pins (pins marked with green color except for pin PA0). These pins must be used to read analog sensors.</li>
<li>PWM pins: 5 pins (pins marked with orange color and a <i>~</i> sign). These pins must be used to write PWM signals.</li>
<li>DAC: Yes</li>
<li>Serial communication: UART, SPI, I2C</li>
<li>Programming: UPDI</li>
<li>Working voltage: 1.8 - 5.5V</li>
</ul>
<p>In order to access the I/O pins and their various functions from within our code, we need to read from/write to <a href="http://www.learningaboutelectronics.com/Articles/IO-registers-embedded-microcontrollers.php">I/O registers</a> of the chip. The programmer uses I/O registers to provide interfaces to the I/O peripherals of the micro-controller. Generally, registers are pins, and for each pin, there is a Data register (PORTx), a Data direction register (DDRx), and a Port input register (PINn). DDRx registers configure pins as INPUT (0) or OUTPUT (1). PINx registers store a value that can be read from the register in case it is defined as INPUT. PORTx registers write the value to pins as HIGH (1) or LOW (0). Each I/O register has a name (used by high-level language/C programmer); I/O address and an SRAM address (used in low-level assembly language).</p>
<p><strong>A summary of ATtiny1614's I/O pins and how to use them:</strong></p>
<ul>
<li>GND: ground</li>
<li>VCC: power supply</li>
<li>PA0 - UDPI/RESET: programming (UPDI interface) & reset pin</li>
<li>PA6 - DAC: digital to analog converter</li>
<li>PA5 - VREF: the voltage reference pin used to enhance the accuracy of analog reading/writing. This pin acts as a very precise analog 'meter stick' against which the incoming analog signal is compared (as in an ADC) or the outgoing analog signal is generated (DAC).</li>
<li>PA1, PA2, PA3, PA4 - MOSI, MISO, SCK, SS: Master Out Slave In, Master In Slave Out, Serial Clock and Slave Select logic signals in <a href="https://en.wikipedia.org/wiki/Serial_Peripheral_Interface">SPI</a> serial communication.</li>
<li>PB0, PB1 - SCL, SDA: Serial Clock and Serial Data lines in <a href="https://en.wikipedia.org/wiki/I%C2%B2C">I2C</a> serial communication. PA1 and PA2 can also be used for the same purpose.</li>
<li>PB2, PB3 - RXD, TXD: Receive and Transmit functions in <a href="https://www.circuitbasics.com/basics-uart-communication/">UART</a> serial communication. PA1 and PA2 can also be used for the same purpose.</li>
</ul>
<p>In the case of ATtiny1614, pin names are of type Pxn, with x being the PORT instance (A, B) and n the pin number. Notation for signals is PORTx, DDRx, and PINn. The numbers 0-10 (marked with orange and blue colors) can also be used while programming in order to indicate the pins we want to refer to.</p>
<p></p>
<h2>Embedded programming</h2>
<h5>Individual assignment - Program the designed ATtiny1614 board</h5>
<h6 id="dice">Arduino IDE + Arduino framework + UPDI programmer + pyupdi</h6>
<p>Let's take a look at my LED dice case as an example of the process: after compiling the <strong>C++ source code</strong> and linking with the <strong>AVR GNU toolchain</strong>, the <strong>Arduino IDE</strong> employs <strong>avrdude</strong> to convert the executable code into a <code>.hex</code> file. The <code>.hex</code> file then is loaded into the ATtiny1614 board via an external <strong>UPDI</strong> programmer (that I made during the 4th week), with the support of <strong>pyupdi</strong> driver.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-06/mill-3.jpg" class="img-fluid w-100"/>
<p><a href="https://github.com/mraardvark/pyupdi">pyupdi</a> can be installed using <a href="https://pypi.org/project/pip/">pip</a>: <kbd>$ pip install https://github.com/mraardvark/pyupdi/archive/master.zip</kbd>.</p>
<p>Below is the C++ code for a LED dice which was written in Arduino IDE using the <a href="https://www.arduino.cc/reference/en/">Arduino framework</a>:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * assign pins to variables 
     */ 
    int LED_1 = 8;
    int LED_2 = 9;
    int LED_3 = 1;
    int LED_4 = 0;
    int LED_5 = 10;
    int LED_6 = 3;
    int LED_7 = 2;
    int BUTTON = 7;

    /* 
     * some useful variables to be used inside functions
     */ 
    int displayTime = 2000;  
    int pendingTime = 100;
    int num = 1;

    /* 
     * setup function, to run once 
     */ 
    void setup() {
        pinMode(BUTTON, INPUT);
        pinMode(LED_1, OUTPUT);
        pinMode(LED_2, OUTPUT);
        pinMode(LED_3, OUTPUT);
        pinMode(LED_4, OUTPUT);
        pinMode(LED_5, OUTPUT);
        pinMode(LED_6, OUTPUT);
        pinMode(LED_7, OUTPUT);
    }

    /* 
     * loop function, to run repeatedly 
     */ 
    void loop() {
        // because I used a pull-up resistor, when the button is pressed, the value is LOW
        if (digitalRead(BUTTON) == LOW) { 
            // get a random number from 1 to 6
            num = random(1, 7);
            // pass the random number to displayRandom() function as an argument  
            displayRandom(num);
        }
        delay(pendingTime);    
    }

    /* 
     * turn certain LEDs of the dice on with a random number from 1 to 6 as the input parameter
     */ 
    void displayRandom(int numDisplayed) {
        // turn all LEDs off before display a new number
        reset();
        if (numDisplayed == 1) {
            digitalWrite(LED_3, HIGH);
        }
        if (numDisplayed == 2) {
            digitalWrite(LED_2, HIGH);
            digitalWrite(LED_6, HIGH);
        }
        if (numDisplayed == 3) {
            digitalWrite(LED_1, HIGH);
            digitalWrite(LED_3, HIGH);
            digitalWrite(LED_4, HIGH);
        }
        if (numDisplayed == 4) {
            digitalWrite(LED_1, HIGH);
            digitalWrite(LED_4, HIGH);
            digitalWrite(LED_5, HIGH);
            digitalWrite(LED_7, HIGH);
        }
        if (numDisplayed == 5) {
            digitalWrite(LED_1, HIGH);
            digitalWrite(LED_3, HIGH);
            digitalWrite(LED_4, HIGH);
            digitalWrite(LED_5, HIGH);
            digitalWrite(LED_7, HIGH);
        }
        if (numDisplayed == 6) {
            digitalWrite(LED_1, HIGH);
            digitalWrite(LED_2, HIGH);
            digitalWrite(LED_4, HIGH);
            digitalWrite(LED_5, HIGH);
            digitalWrite(LED_6, HIGH);
            digitalWrite(LED_7, HIGH);
        } 
        delay(displayTime);
    }

    /* 
     * reset function 
     */ 
    void reset() {
        digitalWrite(LED_1, LOW);
        digitalWrite(LED_2, LOW);
        digitalWrite(LED_3, LOW);
        digitalWrite(LED_4, LOW);   
        digitalWrite(LED_5, LOW);
        digitalWrite(LED_6, LOW);
        digitalWrite(LED_7, LOW);
    }
</code>
</pre>
<p>In order to compile the C++ code, the first step was to install the <a href="https://github.com/SpenceKonde/megaTinyCore">megaTinyCore</a> library which includes Arduino core for the new megaAVR and tinyAVR chips. I navigated to <strong>Files > Preferences </strong> in Arduino IDE and add board definition URLs (JSON files) to the <strong>Add board manager URLs</strong> section. Knowing that I might need to try other boards in the future, I added all URLs provided by the instructors (megaTinyCore, ESP32, ESP8266, NodeMCU). And the next step was going to <strong>Tools > Board > Boards Manager</strong> and installing Arduino AVR Boards, megaTinyCore, esp32, and esp8266 libraries.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/arduinoide.png" class="img-fluid w-100"/>
<p>Now I had to select the correct ATTiny1614 board. Compiled sucessfully (and pay attention to the path of the generated <code>.hex</code> file in the console)!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-06/code-0.png" class="img-fluid w-100"/>
<p>To flash the <code>.hex</code> program to the target board using pyupdi, I need to know the location of the compiled <code>.hex</code> file and the USB port to which my board is connected to. For the first part, I copied the path in the console (see the above image). For the second part, I ran this in Terminal (I was using the computer of my classmate Bruno Molteni with Linux OS since my Mac was running out of battery at the moment) while plugging and unplugging my board: <kbd>$ dmesg -w</kbd>. Finally, with <code>ttyUSB0</code> as the port and <code>/arduino_build_924551/led-dice.ino.hex</code> as the path, I ran this in Terminal: <kbd>$ pyupdi.py -d tiny1614 -c /dev/ttyUSB0 -b 115200 -f /tmp/arduino_build_924551/led-dice.ino.hex -v</kbd>. And here you go my working LED dice!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/zI0-utgwI8o" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-06/code-2.jpg" class="img-fluid w-100"/>
<p></p>
<h6 id="jtag">Arduino IDE + Arduino framework + Arduino Uno as programmer + jtag2updi</h6>
<p>Since I was not having my FTDI cable with me during the lockdown, I tried to look for another way to program my board without the UPDI programmer (I <strong>HATE</strong> it by the way). I then bumped into this <a href="https://www.youtube.com/watch?v=AL9vK_xMt4E">video tutorial</a> which amazingly taught me how upload my program to the ATtiny1614 board using an <strong>Arduino Uno board as UPDI programmer</strong> and <strong><a href="https://github.com/SpenceKonde/jtag2updi">jtag2updi</a></strong>. jtag2updi can be downloaded by cloning the project <kbd>$ git clone https://github.com/SpenceKonde/jtag2updi.git</kbd>. The next step was to open the <code>jtag2updi.ino</code> file in Arduino IDE, and upload it to the Arduino Uno board as a typical Arduino sketch.<p>
<p>Then I wrote another program for an LED race mini game, simple because I would like to test the built-in <strong>Serial Monitor</strong> of Arduino IDE. The rule of the game is quite simple: the LEDs will be turned on one after another, and if we press the button at the same time when the LED in the middle is turned on, we win.</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * assign pins to variables 
     */ 
    int LED_1 = 8;
    int LED_2 = 9;
    int LED_3 = 1;
    int LED_4 = 0;
    int LED_5 = 10;
    int LED_6 = 3;
    int LED_7 = 2;
    int BUTTON = 7;

    /* 
     * setup function, to run once 
     */ 
    void setup() {
        pinMode(BUTTON, INPUT);
        pinMode(LED_1, OUTPUT);
        pinMode(LED_2, OUTPUT);
        pinMode(LED_3, OUTPUT);
        pinMode(LED_4, OUTPUT);
        pinMode(LED_5, OUTPUT);
        pinMode(LED_6, OUTPUT);
        pinMode(LED_7, OUTPUT);
        // begin the serial communication with the baud rate of 115200
        Serial.begin(115200);
    }

    /* 
     * loop function, to run repeatedly 
     */ 
    void loop() {
        Serial.println("Start the loop!");
        check(LED_1);
        check(LED_5);
        check(LED_2);
        check(LED_3);
        check(LED_6);
        check(LED_7);
        check(LED_4);
        delay(2000);   
    }
    
    /* 
     * blink each LED on and check if the button is pressed
     * with the LED pin as the input parameter
     */ 
    void check(int led) {
        digitalWrite(led, HIGH);
        Serial.println(led);
        if (led == 1) {
            if (digitalRead(BUTTON) == LOW && digitalRead(led) == HIGH) {
                Serial.println("Awesome! You win!");
            } 
            else {
                Serial.println("You missed it!");
            }
        } 
        delay(200);
        digitalWrite(led, LOW);
    }
</code>
</pre>
</p>The hardware connection was quite simple: the UPDI data pin of the Arduino Uno board is configured to be pin 6 and it needs to be connected to the UPDI pin of ATtiny1614, the ground to the ground, and the 5V pin of Arduino Uno to the VCC pin of ATtiny1614. I actually had a problem with uploading the sketch:</p> 
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>avrdude: jtagmkII_getsync(): timeout/error communicating with programmer (status -1)</code> 
</pre>
<p>Later I found out that I had to add a capacitor 10uF between the RESET and GND pins of the Arduino Uno board in order to disable the reset that is triggered every time we upload a sketch. We also should be careful with the next step while switching from <strong>uploading <code>jtag2updi.ino</code> to Arduino Uno</strong> to <strong>uploading the program <code>led-race.ino</code> to ATtiny1614 board using jtag2updi programmer</strong>. I forgot to change the board/programmer several times and then having the same error several times.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/jtag-1.png" class="img-fluid w-100"/>
<p>Done uploading! The message below is a spurious warning and can be safely ignored:</p> 
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>avrdude: jtagmkII_initialize(): Cannot locate "flash" and "boot" memories in description</code> 
</pre>
<p>And the LEDs were blinking sequentially!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/3WPSyrly038" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>The game was quite easy though. Here is the sreenshot of what were printed out in the serial monitor (remember to select the correct baud rate by using the same one that was written in the code):</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/jtag-2.png" class="img-fluid w-100"/>
<p></p>
<h5>Additional delivery - Program the Arduino Uno board</h5>
<p>I would like to try writing programs using <strong>plain AVR C</strong> and I/O registers, but a whole LED dice seemed to be a bit complex. That's why I only proceeded with programming a simple LED blink and uploading it to the <strong>Arduino Uno - ATmega328P</strong> board using <strong>Make</strong>.</p> 
<h6>Plain AVR C + Make</h6>
<p><a href="https://en.wikipedia.org/wiki/Make_(software)">Make</a> basically uses its <kbd>$ make</kbd> command to run the following tools: <strong>avr-gcc</strong> (compiling and linking source code into a <code>.elf</code> file), <strong>avr-objcopy</strong> (converting <code>.elf</code> files to <code>.hex</code>), and <strong>avrdude</strong> (uploading/flashing <code>.hex</code> files to the chip). In order to install that AVR toolchain, I ran these commands in Terminal: <kbd>$ brew tap osx-cross/avr</kbd> and <kbd>$ brew install avr-gcc avrdude</kbd>.</p>
<p>The next step was to write the C code. Although I know how to code before, I usually code in class-based and object-oriented language, such as Java and C#. This was the first time I used a procedural programming language - the classic C. Since avr-gcc doesn’t know the Arduino Uno board layout, it only knows the micro-controller that is mounted on it which is the ATmega328P, I had to directly use the hardware I/O registers of the ATmega328P chip. Those information can be found in the <a href="http://ww1.microchip.com/downloads/en/DeviceDoc/Atmel-7810-Automotive-Microcontrollers-ATmega328P_Datasheet.pdf">data sheet</a>. I also explained how to understand the DDR/PORT/PIN registers while reading the <a href="#datasheet">above ATtiny1614 datasheet</a>. My C code was written in 3 styles: plain AVR C, using <a href="https://en.wikipedia.org/wiki/Mask_(computing)">bit masking</a> and using <a href="https://en.wikipedia.org/wiki/Macro_(computer_science)">macros</a>. I tried my best to explain the code below:</p>

<ul class="nav nav-tabs nav-justified d-flex">
  <li class="flex-fill p-2 mr-2 my-2"><a data-toggle="tab" href="#c">Plain AVR C</a></li>
  <li class="flex-fill p-2 mr-2 my-2"><a data-toggle="tab" href="#bits">Bit masking</a></li>
  <li class="flex-fill p-2 mr-2 my-2"><a data-toggle="tab" href="#macros">Macros</a></li>
</ul>
<div class="tab-content">
    <pre id="c" class="tab-pane active bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
    <code>
    /* 
     * AVR C libraries 
     */
    #include &lt;avr/io.h&gt;
    #include &lt;util/delay.h&gt;
    /* 
     * main()  
     */
    int main(void) { 
        /*  
         * the quantity of numbers after 0b are the numbers of PBn pins   
         * since we have 6 PBn pins with PB0 as the first pin and PB5 as the last pin 
         * then PB5 will be the first number after 0b and PB0 as the last number 
         * in this case 0b100000 means setting PB5 as OUTPUT
         */ 
        DDRB = 0b100000; 
        /* 
         * similar to loop() in Arduino 
         */
        while (1) { 
            /*  
             * 0b100000 means setting PB5 as HIGH
             */ 
            PORTB = 0b100000; 
            _delay_ms(500); 
            /*  
             * 0b000000 means setting PB5 as LOW
             */  
            PORTB = 0b000000;  
            _delay_ms(500);
        }
    }
    </code>
    </pre>
    <pre id="bits" class="tab-pane bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
    <code>
    #include &lt;avr/io.h&gt;
    #include &lt;util/delay.h&gt;
    /* 
     * main()  
     */
    int main(void) {
        /* 
         * (1 << DDxn) means setting OUTPUT at DDRxn level  
         * (0 << DDxn) means setting INPUT at DDRxn level  
         * _BV(DDB5) equals (1 << DDB5)
         */
        DDRB |= _BV(DDB5);  
        // or DDRB = (1 << DDB5) | DDRB; 
        // or DDRB |= (1 << DDB5); 
        /* 
         * similar to loop() in Arduino 
         */
        while (1) { 
            /*  
             * (1 << PORTxn) means setting HIGH at PINxn level  
             * | OR operation
             */ 
            PORTB |= _BV(PORTB5);
            // or PORTB = (1 << PORTB5) | PORTB; 
            // or PORTB |= (1 << PORTB5);  
            _delay_ms(500);
            /*  
             * (0 << PORTxn) means setting LOW at PINxn level  
             * & AND operation
             * ~ NOT operation
             */  
            PORTB &= ~_BV(PORTB5); 
            // or PORTB = ~ (1 << PORTB5) & PORTB; 
            // or PORTB = (0 << PORTB5) | PORTB;   
            _delay_ms(500);
        }
    }
    </code>
    </pre>
    <pre id="macros" class="tab-pane bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
    <code>
    #include &lt;avr/io.h&gt;
    #include &lt;util/delay.h&gt;
    /* 
     * Macros 
     */
    #define setOutput(directions, pin) (directions |= (1 << pin))
    #define digitalWriteHigh(port, pin) (port |= (1 << pin))
    #define digitalWriteLow(port, pin) (port &= ~(1 << pin))
    #define time_delay() _delay_ms(500)
    /* 
     * main()  
     */
    int main(void) {
        /* 
         * Using macros defined above: 
         */ 
        setOutput(DDRB, DDB5); 
        /* 
         * similar to loop() in Arduino 
         */
        while (1) {  
            digitalWriteHigh(PORTB, PORTB5);  
            time_delay();
            digitalWriteLow(PORTB, PORTB5);  
            time_delay();
        }
    }
    </code>
    </pre>
</div>
<p></p>

<p>The next step was to compile and flash the program to the board using the commands: <kbd>$ avr-gcc</kbd>, <kbd>$ avr-objcopy</kbd>, and <kbd>$ avrdude</kbd>. In order to deal with multiple configurations and commands, I used a <a href="http://archive.fabacademy.org/archives/2016/doc/makefile.html">Makefile</a> to simplify and automatize the tasks. The <kbd>$ make</kbd> command then will read automatically the Makefile and run them. Generally, most of the parameters and options of the avr-gcc and the avrdude tools for the Uno board can be found in the <code>hardware/arduino/boards.txt</code> file within the Arduino IDE installation directory, and some other information are presented in the <a href="https://www.cs.ou.edu/~fagg/classes/general/atmel/avrdude.pdf">avrdude manual</a>.</p>
<p>Below is the <code>blink.c.make</code> Makefile I wrote based on Guillem's <a href="http://fabacademy.org/2020/labs/barcelona/local/#material/extras/week08/embeddedprogramming">example</a>:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * annotations and variables
     */
    // name of the board 
    MCU=atmega328p
    TARGET_ARCH=-mmcu=$(MCU)

    // name of the C file
    TARGET=blink

    // avr-gcc and its parameters/flags
    CC=avr-gcc
    CPPFLAGS=-mmcu=$(MCU)
    CFLAGS=-Os -g -Wall -I. -DF_CPU=16000000
    LDFLAGS=-g -mmcu=$(MCU) -lm -Wl,--gc-sections -Os

    // avrdude and its parameters/flags
    PGMER=-c arduino -p m328p -b 115200 -P /dev/cu.usbmodem14301
    PGMERISP=-c avrispv2 -P /dev/cu.usbmodem14301
    DUDE=/usr/local/bin/avrdude -V -p $(MCU)

    // files with extension
    C_SRCS= $(TARGET).c
    OBJ_FILES= $(C_SRCS:.c=.out)

    /* 
     * make commands and related original commands
     */
    all:
        $(TARGET).hex

    clean:
        rm -f $(TARGET).elf *.o *.hex  

    %.out: %.c
        $(CC) -c $(CPPFLAGS) $(CFLAGS) $< -o $@

    $(TARGET).elf: $(OBJ_FILES)

        $(CC) $(LDFLAGS) -o $@ $(OBJ_FILES)
    $(TARGET).hex: $(TARGET).elf
        avr-objcopy -j .text -j .data -O ihex $(TARGET).elf $(TARGET).hex
        avr-objcopy -j .eeprom --set-section-flags=.eeprom="alloc,load" --change-section-lma .eeprom=0 -O ihex $(TARGET).elf eeprom.hex

    upload: $(TARGET).hex
        $(DUDE) $(PGMER) -U flash:w:$(TARGET).hex

    size: $(TARGET).elf
        avr-size --format=avr --mcu=$(MCU) $(TARGET).elf
</code>
</pre>
<p>I ran all the commands separately and took a look at the logs to see how Make used the Makefile to compile, link, and flash the program to the chip. At each step, a new file (<code>blink.out</code>, <code>blink.elf</code>, or <code>blink.hex</code>) was generated respectively. The last command uploaded the <code>.hex</code> file to the chip.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/make-1.png" class="img-fluid w-100"/>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/make-2.png" class="img-fluid w-100"/>
<p></p>
<p>The LED blinked on and off as expected!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/J5w5JBzAYlo" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>Let's do something a bit more complicated. I intended to write a simple AVR C program with which the LED fades based on LDR analog value. I followed some AVR C examples (<a href="https://sites.google.com/site/qeewiki/books/avr-guide/analog-input">here</a> and <a href="https://sites.google.com/site/qeewiki/books/avr-guide/pwm-on-the-atmega328">here</a>) in order to know how to <code>analogRead()</code> an LDR and <code>analogWrite()</code> an LED. In short, I had to work with ADC register and conversion, Timer/Counter interrupt handler, <a href="https://en.wikipedia.org/wiki/Pulse-width_modulation">PWM</a> mode, and duty cycle.</p>
<p id="io">And below is the C code:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    #include &lt;avr/io.h&gt;
    #include &lt;util/delay.h&gt;

    /* 
     * main() function, by default returns 0
     */
    int main (void) {
        // setup ADC
        initialize_ADC();
        // setup PWM
        initialize_PWM();

        // initial brightness, duty cycle = 0%
        uint8_t brightness = 0;
        // store LDR readings as volatile data
        volatile uint8_t adc;

        /* 
         * loop() function
         */
        while (1) {
            // divide the reading by 8 and use the result as input duty cycle
            set_PWM(read_ADC()/8);
            _delay_ms(5);
        }
        return 0;
    }
    /* 
     * initialize ADC pin and mode
     */
    void initialize_ADC() {
        // set pin PC0 as INPUT and select ADC(0) as ADC channel
        DDRC |= (1 << PC0); 
        ADMUX = 0;

        // use AVCC as the reference, and adjust for 8 bit resolution
        ADMUX |= (1 << REFS0) | (1 << ADLAR);

        // set prescaler to clock/128 and enable ADC
        ADCSRA |= (1 << ADPS2) | (1 << ADPS1) | (1 << ADPS0) | (1 << ADEN);    
    }

    /* 
     * read and return the LDR
     */
    int read_ADC() {
        int ADCval;
        // start the ADC conversion
        ADCSRA |= (1 << ADSC);    
        // wait for the ADC to finish 
        while (ADCSRA & (1 << ADSC));  
        ADCval = ADCL;
        // update reading
        ADCval = (ADCH << 8) + ADCval;
        // return reading      
        return ADCval;
    }

    /* 
     * initialize PWM pin and mode
     */
    void initialize_PWM() {
        // set pin PD6 (pin 6~) as OUTPUT
        DDRD |= (1 << PD6);
        // use timer0, activate fast PWM mode and non-inverting mode 
        TCCR0A |= (1 << WGM00) | (1 << WGM01) | (1 << COM0A1);
        // set prescaler to clock/8 and start PWM
        TCCR0B |= (1 << CS01);  
    }

    /* 
     * use the input duty cycle to control the brightness of the LED
     */
    void set_PWM(uint8_t duty) {
        OCR0A = duty;
    }
</code>
</pre>
<p>I used the same Makefile and followed the same process to flash the program to the Arduino Uno board (remember to change all file names from <code>blink</code> to <code>fade</code>). Ta-dah! The fading was not really smooth, but my code worked as expected!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/0NuCUubtLlU" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/make-3.png" class="img-fluid w-100"/>
<p></p>
<h5>Additional delivery - Program the ESP32-WROOM Barduino 2.0</h5>
<p>Barduino 2.0 is an ESP32 compatible board designed by our instructor Eduardo Chamorro, and during the last day of the lab before the lockdown, we had a chance to quickly fabricate it. I'd like to play around with the Barduino board in order to try other programming frameworks, especially ESP-IDF and MicroPython.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/barduino.jpg" class="img-fluid w-100"/>
<h6>Arduino IDE + Arduino framework</h6>
<p>In order to test whether the Barduino board worked, I tried a simple blink program written and compiled using Arduino IDE. As shown in the <a href="https://gitlab.fabcloud.org/barcelonaworkshops/barduino-2.0">pinout diagram</a>, the built-in LED is connected to the GPIO-13 pin.</p>
<p>Below is the classic blink program:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    #define BUILTIN_LED 13 
    void setup() {
        pinMode(BUILTIN_LED, OUTPUT);     
    }
    void loop() {
        digitalWrite(BUILTIN_LED, LOW);   
        delay(1000);                     
        digitalWrite(BUILTIN_LED, HIGH);  
        delay(1000);                      
    }
</code>
</pre>
<p>The process was exactly the same as my LED dice, except for choosing the ESP32 Dev Module as the board, <code>/dev/cu.usbserial-D307S8WM</code> as the port (I was using my Mac at the moment). The interesting thing about ESP32 boards is that I don't need an external programmer, since the <a href="https://github.com/espressif/esptool/wiki/ESP32-Boot-Mode-Selection">ESP32 will enter the serial bootloader when GPIO0 is held low on reset</a>, otherwise, it will run the program in flash. Many boards use a button marked "Flash" or "Boot" that pulls GPIO0 low when pressed. In the case of the Barduino board, we have a programmer slider switch that changes the mode of the chip: programming mode or normal execution mode. This is indeed very important since I had to switch on the programming mode and keep pressing the reset button during the flashing process in order to upload my program. Otherwise, I would have the below error:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>A fatal error occurred: Failed to connect to ESP32: Timed out waiting for packet header</code>
</pre>
<p>And it worked as expected! Good soldering!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/barduino-code.png" class="img-fluid w-100"/>
<p>Proof of a working board:</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/OAR-qHhIklg" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<h6 id="pio">PlatformIO + ESP-IDF framework</h6>
<p>PlatformIO is a cross-platform, cross-architecture and multiple framework tool for embedded programming. I personally prefer PlatformIO to Arduino IDE since it works effectively in Terminal (they have an IDE, but I simply used <a href="https://www.vim.org/">vim</a> as the editor and directly used the command lines). Besides, PlatformIO offers integrated libraries for a wide range of boards and frameworks, including Arduino framework and <a href="https://docs.platformio.org/en/latest/frameworks/espidf.html">ESP-IDF</a> framework. Thanks to PlatformIO, I didn't have to go through all the complicated setup and configuration steps. All that I had to do to set up the ESP-IDF development environment was:</p>
<ul>
<li><kbd>$ brew install platformio</kbd>: install PlatformIO</li>
<li><kbd>$ mkdir barduino-blink-platformio</kbd> and <kbd>$ cd barduino-blink-platformio</kbd>: create a folder for my project and go inside the folder (meh)</li>
<li><kbd>$ pio boards esp32-</kbd>: browse the libraries and look for the ESP32 board. Here I saw many boards, and the <i>esp32-devkitlipo</i> seemed to be the one I was looking for.</li>
<li><kbd>$ pio project init --board esp32-devkitlipo</kbd>: initialize the project. Now if I run <kbd>$ ls</kbd> I would see a typical ESP-IDF structure with an <code>empty.c</code> file and <code>platformio.ini</code> file (all that I should care about unless I want to rename the C file).</li>
</ul>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/platformio-0.png" class="img-fluid w-100"/>
<p>The next step was to put some configurations into the <code>platformio.ini</code> file. I haven't read about it carefully, but the <a href="https://www.setra.com/blog/what-is-baud-rate-and-what-cable-length-is-required-1">baud rate</a> 115200 was recommended by many tutorials.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/platformio-1.png" class="img-fluid w-100"/>
<p>And below is the C code to put into the <code>empty.c</code> file, with some not-so-useful comments (yes you're right, I'm the type of developer that can comment something like <code>/* Lorem ipsum dolor sit amet */</code>):</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * C libraries 
     */
    // define several macros and various functions for performing input and output
    #include &lt;stdio.h&gt; 
    // FreeRTOS is a real-time operating system kernel for embedded devices
    #include &lt;freertos/FreeRTOS.h&gt; 
    #include &lt;freertos/task.h&gt; 
    // ESP-IDF driver used to setup/configure GPIO's mode, interupt trigger type, and many more...
    #include &lt;driver/gpio.h&gt; 
    // auto-generated ESP-IDF config file containing saved settings
    #include &lt;sdkconfig.h&gt; 

    /* 
     * assign pins to variables 
     */ 
    #define BLINK_GPIO 13 

    void app_main(void){
        gpio_pad_select_gpio(BLINK_GPIO);
        /* 
         * set the GPIO as an output - similar to setup() in Arduino 
         */
        gpio_set_direction(BLINK_GPIO, GPIO_MODE_OUTPUT);
        /* 
         * similar to loop() in Arduino 
         */
        while(1) {
            gpio_set_level(BLINK_GPIO, 0);
            /*
             * FreeRTOS function to specify a time 
             * at which the task wishes to unblock relative to 
             * the time at which vTaskDelay() is called
             */
            vTaskDelay(1000/portTICK_PERIOD_MS); 
            gpio_set_level(BLINK_GPIO, 1);
            vTaskDelay(1000/portTICK_PERIOD_MS);
        }
    }
</code>
</pre>
<p>In order to compile and flash the program to the chip, these commands were executed:</p>
<ul>
<li><kbd>$ pio run</kbd>: compile the C source code. In order to understand what was going on behind the scenes, I analyzed the log a bit. Basically, by running <kbd>$ pio run</kbd>, PlatformIO helps us to install and run some useful tools: <strong>esp-idf</strong> framework, <strong>toolchain-xtensa32</strong> (ESP32 toolchain to compile code), <strong><a href="https://en.wikipedia.org/wiki/CMake">CMake</a></strong> and <strong><a href="https://en.wikipedia.org/wiki/Ninja_(build_system)">Ninja</a></strong> (build systems similar to Make), and <strong><a href="https://github.com/espressif/esptool">esptool.py</a></strong> (flashing utility similar to pyupdi).</li>
<li><kbd>$ pio run -t upload</kbd>: flash the program to the chip using esptool.py commands</li>
</ul>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/platformio-2.png" class="img-fluid w-100"/>
<p>The LED blinked on and off again, but it's unnecessary to keep posting another video with the same content ;)</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/platformio-3.png" class="img-fluid w-100"/>
<p></p>
<h6>MicroPython + esptool.py + ampy</h6>
<p>MicroPython is an open-source Python interpreter that runs on small embedded development boards. With MicroPython we can write easy-to-read Python code instead of having to use brain-tangled languages like C/C++ (I don't know why many people consider Python as simpler than C++, to me they're quite the same). Actually, the nice thing about MicroPython is that it has an interactive REPL feature which is a simple computer programming environment that takes single user inputs, evaluates them piecewise, and returns the result to the user. I definitely had to try MicroPython, since they said using MicroPython is a great way to get the most of an ESP32 board, and vice versa, the ESP32 chip is a great platform for using MicroPython.</p>
<p>To use MicroPython, the first thing was to download the most recent MicroPython firmware <code>.bin</code> file which would be loaded onto the ESP32 board. I followed this <a href="http://docs.micropython.org/en/latest/esp32/tutorial/intro.html#esp32-intro">tutorial</a> and downloaded the firmware <a href="https://micropython.org/download#esp32">here</a>.</p>
<p>It is recommended to first erase the entire flash of the ESP32 board before putting on the new MicroPython firmware. To do that, first I had to install <a href="https://github.com/espressif/esptool">esptool.py</a> by running <kbd>$ pip install esptool</kbd> in Terminal. Then these commands were executed:</p>
<ul>
<li><kbd>$ esptool.py --port /dev/cu.usbserial-D307S8WM erase_flash</kbd>: erase the entire flash</li>
<li><kbd>$ esptool.py --chip esp32 --port /dev/cu.usbserial-D307S8WM --baud 112500 write_flash -z 0x10 esp32-idf3-20200327-v1.12-310-g9418611c8.bin</kbd>: deploy the new firmware</li>
</ul>
<p>Actually I had an error trying to deploy the firmware while following the exact command line mentioned in the tutorial <kbd>$ esptool.py --chip esp32 --port /dev/cu.usbserial-D307S8WM --baud 112500 write_flash -z 0x1000 esp32-idf3-20200327-v1.12-310-g9418611c8.bin</kbd>:</p> 
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>A fatal error occurred: Failed to connect to ESP32: Invalid head of packet (0x30)</code>
</pre>
<p>After checking carefully again, I assumed the problem was not the port or the firmware's path, and after bumping into this <a href="https://github.com/espressif/esptool/wiki/ESP32-Boot-Mode-Selection">section</a>, I found out that I should use the correct address of the GPIO0 pin, which is <strong>0x10</strong> instead of <strong>0x1000</strong>. And the MicroPython firmware was successfully deployed!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-08/micropython-1.png" class="img-fluid w-100"/>
<p>Below is the Python code I wrote in a new <code>main.py</code> file.</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    // control GPIO pins 
    import machine
    // give access to underlying interpreter values 
    import sys
    // tell the program to wait for a specified amount of time
    import time

    /*
     * similar to setup() in Arduino
     */
    led = machine.Pin(13, machine.Pin.OUT)
    
    /* 
     * similar to loop() in Arduino
     */
    while True:
        // print something to debug in the serial monitor
        print("LED ON") 
        led.value(1)
        time.sleep(0.5)
        print("LED OFF")
        led.value(0)
        time.sleep(0.5)
</code>
</pre>
<p>MicroPython is a little different at first glance compared to all of the above workflows. With MicroPython, the language may have come with the hardware, and we have the option of working with it interactively via its REPL (read-evaluate-print-loop) feature. For most MicroPython boards, we can access the MicroPython REPL over USB serial connection or Wifi. To get a serial REPL on a Mac, I entered the built-in serial terminal and connect to the board by running: <kbd>$ screen /dev/cu.usbserial-D307S8WM 115200</kbd>.</p>
<p>In order to upload the <code>main.py</code> program to ESP32 chips, I used a tool called <a href="https://github.com/scientifichackers/ampy">ampy</a>, which is a Python script that can read and write files to a MicroPython board over a serial connection. I followed this <a href="https://pythonforundergradengineers.com/upload-py-files-to-esp8266-running-micropython.html">tutorial</a> and ran <kbd>$ pip install adafruit-ampy</kbd> to install ampy.</p>
<p>At this point I faced 2 problems:</p>
<ul>
<li>I couldn't see <i>>>> prompt</i> while entering the serial terminal. After pressing Control-C a couple times to interrupt any running program on the board, it still didn't work.</li>
<li>All ampy commands took forever to execute. I tried running <kbd>$ ampy --port /dev/cu.usbserial-D307S8WM ls</kbd> to list the available files, <kbd>$ ampy --port /dev/cu.usbserial-D307S8WM get boot.py</kbd> to read the <code>boot.py</code> config file, and <kbd>$ ampy --port /dev/cu.usbserial-D307S8WM put main.py</kbd> to upload the <code>main.py</code> blink program to the chip. Both commands took a long time without finishing execution or returning any errors.</li>
</ul>
<p>After researching for a while, I noticed that <a href="https://github.com/scientifichackers/ampy/issues/71">ampy is no longer maintained by Adafruit</a> which is a piece of bad news for my previous attempts since issues will no longer be solved. The biggest downside of MicroPython could be a small community where there are only a few example codes or questions solved online. I assumed that the problem could be because I didn't enter the programming mode of the board correctly, but, losing motivation, I decided to leave it like that for this week. I might get back to MicroPython in the future, but for now I'm pretty sure I will continue using Arduino or ESP-IDF for upcoming assignments.</p>
<p></p>
<h5 id="group">Group assignment - Compare tinyAVR, megaAVR and Xtensa families</h5>
<p>Or should I call it the <strong>TL,DR;</strong> section?</p>
<h6>Comparing the micro-controllers</h6>
<p>Due to the COVID-19 lockdown, I couldn't work with my classmates for the group assignment. Hence, again, I tried to fulfill it myself by comparing my experiences reading the data sheets and programming the ATtiny1614, ATmega328P and ESP32-WROOM boards (we classmates were working with similar MCUs anyway).</p>
<p></p>
<div style="overflow-x: auto;">
<table class="table">
  <thead class="thead-light">
    <tr>
      <th scope="col">MCU - MCU family</th>
      <th scope="col">Working voltage</th>
      <th scope="col">Flash memory</th>
      <th scope="col">EEPROM</th>
      <th scope="col">RAM</th>
      <th scope="col">GPIO pins</th>
      <th scope="col">ADC</th>
      <th scope="col">DAC</th>
      <th scope="col">PWM</th>
      <th scope="col">Serial protocol</th>
      <th scope="col">Wireless communication</th>
      <th scope="col">Programming protocol/interface</th>
      <th scope="col">External programmer required</th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th scope="row">ATtiny1614 - Atmel tinyAVR</th>
      <td>1.8 - 5.5V</td>
      <td>16 kiB</td>
      <td>256 bytes</td>
      <td>2 kiB</td>
      <td>11 usable</td>
      <td>10 channels</td>
      <td>Yes</td>
      <td>5 channels</td>
      <td>UART, SPI, I2C</td>
      <td>No</td>
      <td>UPDI</td>
      <td>Yes</td>
    </tr>
    <tr>
      <th scope="row">ATmega328P - Atmel megaAVR</th>
      <td>1.8 - 5.5V</td>
      <td>32 KiB</td>
      <td>1 KiB</td>
      <td>2 kiB</td>
      <td>23 usable</td>
      <td>8 channels</td>
      <td>No</td>
      <td>6 channels</td>
      <td>UART, SPI, I2C</td>
      <td>No</td>
      <td>AVRISP, ArduinoUSP, USBasp</td>
      <td>Yes (in general) - No (in case of Arduino Uno board)</td>
    </tr>
    <tr>
      <th scope="row">ESP32-WROOM - Xtensa ESP32</th>
      <td>2.3 - 3.6V</td>
      <td>16 MB</td>
      <td>512 bytes</td>
      <td>520 KiB</td>
      <td>34 usable</td>
      <td>18 channels</td>
      <td>Yes</td>
      <td>16 channels</td>
      <td>SPI, UART, I2C, I2S, SDIO, Ethernet</td>
      <td>Wifi, Bluetooth, IR, RF</td>
      <td>UART bootloader, OTA</td>
      <td>No</td>
    </tr>
  </tbody>
</table>
</div>
<p></p>
<h6>Comparing development workflows</h6>
<p>To make this part more like a "group" assignment, I included here the development workflows that my classmates went through in addition to what I did. All of them tried many workflows and made amazing programs, but I limited the list to workflows that are different from mine.</p>
<p></p>
<div style="overflow-x: auto;">
<table class="table">
  <thead class="thead-light">
    <tr> 
      <th scope="col">MCU</th>
      <th scope="col">Student</th>
      <th scope="col">Editor/IDE/IDP</th>
      <th scope="col">Language</th>
      <th scope="col">Framework</th>
      <th scope="col">Configuration file</th>
      <th scope="col">Compiler/Linker</th>
      <th scope="col">Flashing tool/utility</th>
      <th scope="col">Hardware ISP</th>
      <th scope="col">Serial monitor</th>
    </tr>
  </thead>
  <tbody>
    <tr>
      <th scope="row">ATtiny1614</th>
      <td>Myself</td>
      <td>Arduino IDE</td>
      <td>C++</td>
      <td>Arduino</td>
      <td>No</td>
      <td>avr-gcc + ld (Arduino IDE)</td>
      <td>pyupdi</td>
      <td>UPDI adapter</td>
      <td>Arduino IDE's Serial Monitor</td>
    </tr>
    <tr>
      <th scope="row">ATtiny1614</th>
      <td>Myself</td>
      <td>Arduino IDE</td>
      <td>C++</td>
      <td>Arduino</td>
      <td>No</td>
      <td>avr-gcc + ld (Arduino IDE)</td>
      <td>avrdude (Arduino IDE) + jtag2updi</td>
      <td>Arduino Uno</td>
      <td>Arduino IDE's Serial Monitor</td>
    </tr>
    <tr>
      <th scope="row">ATmega328P (Arduino Uno)</th>
      <td>Myself</td>
      <td>vim</td>
      <td>C</td>
      <td>No</td>
      <td>Makefile</td>
      <td>avr-gcc + arv-objcopy (GNU Make)</td>
      <td>avrdude (GNU Make)</td>
      <td>No</td>
      <td>Mac OS X's screen</td>
    </tr>
    <tr>
      <th scope="row">ESP32-WROOM</th>
      <td>Myself</td>
      <td>MicroPython's REPL + vim</td>
      <td>Python</td>
      <td>No</td>
      <td><code>boot.py</code></td>
      <td>MicroPython</td>
      <td>esptool.py + ampy</td>
      <td>No</td>
      <td>Mac OS X's screen</td>
    </tr>
    <tr>
      <th scope="row">ESP32-WROOM</th>
      <td>Myself</td>
      <td>PlatformIO + vim</td>
      <td>C</td>
      <td>ESP-IDF</td>
      <td><code>platformio.ini</code> and <code>CMakeLists.txt</code></td>
      <td>CMake + Ninja (PlatformIO)</td>
      <td>esptool.py (PlatformIO)</td>
      <td>No</td>
      <td>PlatformIO's Miniterm-based serial monitor</td>
    </tr>
    <tr>
      <th scope="row">ESP32-WROOM</th>
      <td><a href="http://fabacademy.org/2020/labs/barcelona/students/benjamin-scott/week-8.html">Benjamin Scott</a></td>
      <td>GNU nano</td>
      <td>C</td>
      <td>ESP-IDF</td>
      <td><code>CMakeLists.txt</code></td>
      <td>CMake + Ninja (idf.py)</td>
      <td>esptool.py (idf.py)</td>
      <td>No</td>
      <td>IDF Monitor</td>
    </tr>
    <tr>
      <th scope="row">ATSAMD21 (Circuit Playground Express)</th>
      <td><a href="http://fabacademy.org/2020/labs/barcelona/students/antoine-jaunard/fabac-assignments-embedded-programming.html">Antoine Jaunard</a></td>
      <td>PlatformIO + Neovim</td>
      <td>C++</td>
      <td>Arduino</td>
      <td><code>platformio.ini</code></td>
      <td>gcc-arm-embedded (PlatformIO)</td>
      <td>SAM-BA (PlatformIO)</td>
      <td>No</td>
      <td>PlatformIO's Miniterm-based serial monitor</td>
    </tr>
  </tbody>
</table>
</div>
<p></p>
<h2>Conclusion</h2>
<p>In general, it was a hardcore but fun week and I understood better some terms and concepts I heard before. I really enjoyed working with the ESP32 micro-controller, and its ESP8266 predecessor seems to be good too. They're cheap, they have many GPIO pins, and they don't need an external ISP in order to be programmed. Something similar to the Arduino Uno board could also be a good choice. PlatformIO is also really amazing! It supports many boards and platforms, includes many toolchains and frameworks. In case I need help from the active Arduino communities, I can easily refer to similar issues and apply solutions using PlatformIO. It also has a very stable Miniterm-based built-in serial monitor, which is much better than Arduino IDE's Serial Monitor and Mac OS X's screen. The workflow was also simple and straightforward, and I can do everything using command lines. I will definitely use PlatformIO and Xtensa chips for upcoming assignments!</p>
<p></p>

<div class="container w-100 text-center py-4">
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-07-computer-controlled-machining.html">Previous Assignment</a>
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-09-input-devices.html">Next Assignment</a>
</div>
