---
layout: assignment
title: Input Devices
thumb: 10.jpg
assignment: add a sensor to a designed micro-controller board and read it (individual); to probe an input device's analog levels and digital signals (in-group). 
active: 1
---

<p class="font-italic font-weight-bold">* All C++ and KiCad files can be downloaded <a href="https://gitlab.fabcloud.org/academany/fabacademy/2020/labs/barcelona/students/tue-ngo/tree/master/assets/img/input">here</a>.</p>

<p class="font-italic font-weight-bold">* This week's <a href="#group">group assignment</a> is to probe an input device's analog levels and digital signals.</p>

<p class="font-italic font-weight-bold">* My plan for this week's progress was to make <a href="#diy">DIY sensors</a> using basic components and test them in order to understand how sensors work and to <a href="#design">design a board with built-in sensors</a>. The board will be fabricated after the lockdown.</p>

<p>In an effort to maintain the classes during the COVID-19 pandemic, we were provided by our instructors a basic kit including an Arduino Uno board, an HC-SR04 ultrasonic 2 - 400cm distance sensor, a light-dependent resistor (LDR), some LEDs, some capacitors and some resistors.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/sensors.jpg" class="img-fluid w-100"/>
<p>I want to design a multi-sensor shield for an ESP32 board as my individual assignment as per the requirements of this week. Both the ESP32 board and the multi-sensor shield could be used for my Final Project and would be fabricated after the lockdown. Before being able to add sensors to a designed board, my plan for the lockdown period was to test the HC-SR04 ultrasonic proximity sensor and make a darkness/proximity sensor using the LDR and the LEDs. I want to work on those sensors for my Final Project so reading their data could be a good exercise.</p>

<h2>Reading input signals</h2>

<h5 id="group">Group assignment - Probe the KY-037 sound sensor</h5>
<p>As defined in the <a href="http://fabacademy.org/2020/docs/fabacademy-assessment/input_devices.html">Assessment Guide</a>, we have to measure, observe or monitor signals with test probes of a physical device. I only had a cheap multimeter with me and I tried to fulfill the requirements by probing the KY-037 sound sensor because it can produce both analog and digital outputs. Analog sensors continuously produce analog voltage signals as outputs and detect any proportional value between 100% (1.8V, 3.3V, 5V) and 0% (0V). A digital sensor only detect two possible status: if it is working at 100% (above a certain threshold value) or at 0% (below the threshold value). The signal measured is directly converted into digital signal output (0 and 1) inside the digital sensor itself. A detailed comparison can be found <a href="https://learn.sparkfun.com/tutorials/analog-vs-digital/all">here</a>.</p> 
<p>The hardware connection was quite simple: I used jumper wires to connect the VCC pin of the sensor to the 5V pin of the Arduino Uno board, the GND pin to the GND pin, the A0 pin to the analog pin A0 and the D0 pin to the digital pin marked with number 3.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/group-1.jpg" class="img-fluid w-100"/>
<p>It's time to turn some music on! I could probe voltage values ranging from 0.22 - 0.28V by poking at GND and A0. Voltage values of 4.8 - 4.98V were measured while poking at GND and digital pin 3. They seemed plausible to me.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/group-2.jpg" class="img-fluid w-100"/>
<p>In order to see printed values of the digital and analog signals, I used Arduino IDE and its Serial Monitor. I printed the analog values (0 - 1023), the converted voltage from the analog values (0 - 5V), and the digital values (0 or 1):</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    void setup() {
        pinMode (A0, INPUT);
        pinMode (3, INPUT);
        Serial.begin(115200); 
    }

    void loop() {
        Serial.print(analogRead(A0));
        Serial.print(" ");
        Serial.print(analogRead(A0) * (5.0 / 1023.0));
        Serial.print(" ");
        Serial.println(digitalRead(3));
        delay(1000);
    }
</code>
</pre>
<p>As per the <a href="https://datasheet4u.com/datasheet-pdf/Joy-IT/KY-037/pdf.php?id=1402047">data sheet</a>, the signal will be inverted, which means the louder the noise detected, the lower the value at the analog output. The sensor was super sensitive. I had to turn off the music, pressed the microphone to my pillow in order to get a higher analog value (over 500), and I could not manage to get a digital value of 0, even if I tried to keep it completely silent and reduced sensivity by rotating the potentiometer.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/group-3.png" class="img-fluid w-100"/>
<p>Here I can detected similar analog voltage values as when I probed with the multimeter (~0.26V). I will try to do this group assignment properly by using an oscilloscope when we get back to the lab.</p>
<p></p>

<h5>Additional test - Read the HC-SR04 ultrasonic proximity sensor</h5>
<p>I need a distance sensor for my Final Project, so I scrolled through this <a href="https://www.sparkfun.com/distance_sensor_comparison_guide">article</a> and this <a href="https://www.instructables.com/id/Selecting-the-Right-Sensor-for-Arduino-Projects/">instruction.</a> The main reason why I chose to go ahead with the ultrasonic proximity sensor is that I want to play with a medium range of distance (8 - 30cm), I need something small and cheap (less than 10€) that consumes less current, and the sensor must be able to work well under low light conditions. Most of the other options are either expensive, have too narrow or too wide range, have large footprints, or demand high continuous current.</p> 
<p>Ultrasonic proximity sensors work by emitting sound waves that travel through the air with the speed of sound, roughly 0.034 cm/s. If there is an object in front of the sensor, the sound waves get reflected back and the receiver of the ultrasonic sensor detects them. By measuring how much time passed between sending and receiving the sound waves, the distance between the sensor and the object can be calculated. The formula I used was <i class="font-weight-bold">distance = (travel time * speed of sound)/2</i>.</p> 
<p id="pygame">In order to know the required voltage of the HR-SC04 sensor I had with me, I referred to this <a href="https://www.makerguides.com/wp-content/uploads/2019/02/HCSR04-Datasheet.pdf">data sheet</a>. I wanted to use the Barduino ESP32 board, but the sensor requires a working voltage of 5V which exceeds the working voltage of the Barduino board. Hence, I used the provided Arduino Uno board instead and followed this <a href="https://www.makerguides.com/hc-sr04-arduino-tutorial/">tutorial</a> in order to understand how to use Arduino to read the data from the ultrasonic sensor. I also tried connecting the sensor to my custom ATtiny1614 board which was designed in the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-06-electronics-design.html#design">6th week</a> and using Arduino Uno to program it (similar to what I did <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-08-embedded-programming.html#jtag">last week</a>). Below are 2 programs I used for both boards:</p> 

<ul class="nav nav-tabs nav-justified d-flex">
  <li class="flex-fill p-2 mr-2 my-2"><a data-toggle="tab" href="#uno">Arduino Uno</a></li>
  <li class="flex-fill p-2 mr-2 my-2"><a data-toggle="tab" href="#dice">ATtiny1614</a></li>
</ul>
<div class="tab-content">
    <pre id="uno" class="tab-pane active bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
    <code>
    /* 
     * Arduino framework must be explicitly included 
     */  
    #include &lt;Arduino.h&gt;
    /* 
     * useful variables 
     */ 
    // pin variables
    int trig = 11;
    int echo = 12;
    int led = 13;
    // variables to store values after reading data from the sensor
    long unsigned duration = 0;
    int distance = 0;
    // trying basic threshold DSP, when the distance is greater than 10cm, do something
    const int threshold = 10;
    /* 
     * setup function, to run once 
     */ 
    void setup() {
        pinMode(trig, OUTPUT); 
        pinMode(echo, INPUT);
        pinMode(led, OUTPUT); 
        // begin the serial communication with the baud rate of 115200
        Serial.begin(115200);
    }
    /* 
     * loop function, to run repeatedly 
     */ 
    void loop() {
        // the LED will always be turned on
        digitalWrite(led, HIGH);
        /* 
         * trigger the sensor 
         */ 
        // clear the trigger pin
        digitalWrite(trig, LOW);
        delayMicroseconds(5);
        // trigger the sensor by setting the trigger pin high for 10 microseconds
        digitalWrite(trig, HIGH);
        delayMicroseconds(20);
        digitalWrite(trig, LOW);
        /* 
         * read and process data 
         */
        // pulseIn() measures the time period of a pulse (either HIGH or LOW) input signal
        // if value is HIGH, pulseIn() waits for the pin to go HIGH, starts timing, then waits for the pin to go LOW and stops timing 
        // pulseIn() returns the duration in microseconds 
        duration = pulseIn(echo, HIGH);
        // and calculate the distance based on the formula: distance = (travel time * speed of sound)/2
        distance = (duration * 0.034)/2;
        /* 
         * blink the LED when the distance is greater than 10cm 
         */
        if (distance >= threshold) {
            digitalWrite(led, LOW);
        }   
        /* 
         * print the data on the serial monitor 
         */ 
        // Serial.print("Duration: ");
        // Serial.println(duration);
        // Serial.print("Distance: ");
        // Serial.println(distance);
        /* 
         * print the data on the serial plotter  
         * values were modified for a nicer graph
         */
        Serial.print(threshold * 100);
        Serial.print(" ");
        Serial.print(digitalRead(led) * 100);
        Serial.print(" ");
        Serial.print(duration);
        Serial.print(" ");
        Serial.println(distance * 100);
        delay(1000);
    }
    </code>
    </pre>
    <pre id="dice" class="tab-pane bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
    <code>
    /* 
     * new pin variables
     */
    int trig = 4;
    int echo = 5;
    long unsigned duration = 0;
    int distance = 0;
    /* 
     * set the LED pins to OUTPUT
     */
    void setup() {
        pinMode(trig, OUTPUT); 
        pinMode(echo, INPUT);
        pinMode(0, OUTPUT);
        pinMode(1, OUTPUT);
        pinMode(2, OUTPUT);
        pinMode(3, OUTPUT);
        pinMode(8, OUTPUT);
        pinMode(9, OUTPUT);
        pinMode(10, OUTPUT);
        Serial.begin(112500);
    }
    void loop() {
        reset();
        digitalWrite(trig, LOW);
        delayMicroseconds(5);
        digitalWrite(trig, HIGH);
        delayMicroseconds(20);
        digitalWrite(trig, LOW);
        duration = pulseIn(echo, HIGH);
        distance= (duration * 0.034)/2;
        /* 
         * call a function to display certain numbers according to the distance
         */
        displayNumber(distance);
        delay(200);
    }
    /* 
     * conditions to display numbers
     */
    void displayNumber(int distance) {
        if (distance >= 5 && distance < 10) {
            digitalWrite(1, HIGH);
        }
        else if (distance >= 10 && distance < 15) {
            digitalWrite(2, HIGH);
            digitalWrite(10, HIGH);
        } 
        else if (distance >= 15 && distance < 20) {
            digitalWrite(1, HIGH);
            digitalWrite(2, HIGH);
            digitalWrite(10, HIGH);
        }
        else {
            reset(); 
        }
        delay(1000);
    }
    void reset() {
        digitalWrite(0, LOW);
        digitalWrite(1, LOW);
        digitalWrite(2, LOW);
        digitalWrite(3, LOW);
        digitalWrite(8, LOW);
        digitalWrite(9, LOW);
        digitalWrite(10, LOW);
    }
    </code>
    </pre>
</div>
<p></p>

<h6>Arduino Uno board</h6>
<p>The hardware connection was quite simple: I used jumper wires to connect the VCC pin of the HC-SR04 sensor to the 5V pin of the Arduino Uno board, the GND pin to the GND pin, the Echo pin to the digital pin PB4 (marked with number 12) and the Trig pin to the digital pin PB3 (marked with number 11).</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/ultrasonic-1.jpg" class="img-fluid w-100"/>
<p>PlatformIO was used for the whole process since I had a good experience using it during the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-08-embedded-programming.html#pio">previous assignment</a>. This time, I tried using Arduino framework:</p>
<ul>
<li><kbd>$ pio project init --board uno</kbd>: initialize the project</li>
<li>Create a new <code>main.cpp</code> file into the <code>src</code> folder and input the source code</li>
<li><kbd>$ pio run -t upload</kbd>: compile and upload the program</li>
</ul>
<p>To see the duration and distance printed in the built-in serial monitor of PlatformIO, I ran <kbd>$ pio device monitor</kbd>. The results were more or less accurate, but sometimes when I directly touched the receiver of the sensor, or when my hand was too far away, it returned weird values, such as <strong>2240</strong> something. It could be because the sensor can only recognize distance within the range of 2 - 400cm, however, as per my observations, the values seemed to be more accurate within the range of 2 - 60cm.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/ultrasonic-2.png" class="img-fluid w-100"/>
<p>In order to see a graph of the digital and analog signals, I had to use Arduino IDE and its Serial Plotter. The values were modified a bit in order to be displayed as a nicer graph. Here I have the blue line as the 10cm threshold * 100, the red line as the digital state of the LED * 100, the green line as the measured duration, and the orange line as the measured distance * 100:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/ultrasonic-5.png" class="img-fluid w-100"/>
<p>Below is the video testing whether the LED will blink according to the threshold value of 10cm:</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/Q2iX2uzZttg" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>

<h6>ATtiny1614 custom board</h6>
<p>Since the custom board has more LEDs, I decided to play further with threshold processing. Based on the code I wrote, the board supposed to display certain patterns according to different ranges of distance.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/ultrasonic-3.jpg" class="img-fluid w-100"/>
<p>This time, the hardware connection was not that easy. Indeed, it was very complex. In summary, these were the required connections for a working Frankenstein mess:</p>
<ul>
<li>Pin 6 of the Arduino Uno board should be connected to the UPDI pin of the ATtiny1614 board, adding a 5K resistor. This was to program the board using Arduino Uno as the UPDI programmer.</li>
<li>A capacitor 10uF was added between GND and RESET pins of the Arduino Uno board in order to disable auto-reset.</li>
<li>All GND pins should be connected together: the GND pin of the Arduino Uno board was connected to GND pin/UPDI header of the ATtiny1614 board, and GND pin/FTDI header of the ATtiny1614 board was connected to the GND pin of the sensor.</li>
<li>All VCC pins should be connected together: the 5V pin of the Arduino Uno board was connected to the breadboard, and the VCC pins of both the ATtiny1614 board and the sensor were connected to it.</li>
<li>I connected 2 UART pins of the ATtiny board to the echo and trigger pins of the sensor. We should pay attention to the input/output (-/+) directions while plugging the pins to the breadboard.</li>
</ul>
<p>The board was not working and I realized the reason was because I connected the UART pins with the ultrasonic sensor, and then the ATtiny1614 board was confused about the purposes of pin 4 and pin 5. I had to go to <strong>Tool > UART pins</strong> in Arduino IDE and changed the UART pins from pin 4, 5 to pin 8, 9.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/ultrasonic-4.png" class="img-fluid w-100"/>
<p>I intended to use all 7 LEDs to display 7 patterns based on 7 ranges of distance. However, because pin 8 and 9 were already used as UART pins, weird patterns were displayed. In the end, I can only display 3 different LED patterns based on 3 ranges of distance. Finally, the board behaved the way I want it to! What I learned from this test actually helped me a lot with understanding how to design a board with built-in sensors or a breakout board! Very helpful exercise!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/xd7EVw5dpII" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>

<h2 id="diy">Making and testing DIY sensors</h2>
<h5>Additional test - LDR darkness/proximity sensor</h5>
<p id="npn">An LDR is a component that has a variable resistance that changes with the light intensity that falls upon it: in the dark, its resistance increases; however in light, its resistance decreases. The resistance of an LDR may typically have around 5KΩ in daylight and 1MΩ in darkness. <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-08-embedded-programming.html#io">Last week</a> I used Arduino code to turn on the LED in accordance with the analog values from the LDR. This week, I intended to use a transistor as a switch. In short, a transistor conducts current across the collector-emitter path only when a voltage is applied to the base. When no base voltage is present, the switch is off. When a base voltage is present, the switch is on. By using the LDR to bias a transistor, we can make a transistor switch power to a load depending upon the light conditions.</p>
<p>I followed this <a href="https://www.instructables.com/id/Transistor-DarkLight-Sensor/">tutorial</a> to make a DIY darkness sensor using the LDR and a 2N222 transistor. An LED added to the circuit would be turned on or off depends on the light condition. The dark sensor circuit works in the way that if no light falls on the LDR, the LDR's resistance increases. So the voltage at the base of the transistor increases when the brightness of light decreases, and once it gets past the minimum threshold voltage required at the base of the transistor, it turns on the LED. I was also thinking that this concept could be used to understand how an IR proximity sensor works, more or less. Detailed schematic diagram is described in the tutorial.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/light-1.jpg" class="img-fluid w-100"/>
<p>Below is the code I used to read signals in Arduino IDE. The sensor pin (A0) was connected to one leg of the LDR (the leg that was connected to the middle leg of the transistor):</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    int sensorPin = A0; 
    int sensorValue = 0; 

    void setup() {
        Serial.begin(112500); 
    }

    void loop() {
        sensorValue = analogRead(sensorPin); 
        Serial.println(sensorValue);
        delay(1000);
    }
</code>
</pre>
<p>Below is the voltage data printed out in the serial monitor, the closer to the LDR my finger was, the higher the number:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/light-2.png" class="img-fluid w-100"/>
<p>Analog values displayed in the serial plotter:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/light-3.png" class="img-fluid w-100"/>
<p>Below is the demo video of how the circuit worked:</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/-9uvopm1abQ" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>

<h5>Additional test - LDR light sensor</h5>
<p>I followed the same tutorial and used the same code to work on the light sensor. The only difference between the 2 circuits was the placement of the LDR and the 100KΩ resistor. The light sensor circuit works in the opposite of what we saw with the dark sensor circuit: when the brightness of light increases, the LDR's resistance reduces and so the voltage at the base of transistor increases. Once this voltage increases above the required threshold voltage at the base, the LED turns on. Detailed schematic diagram is described in the tutorial.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/dark-1.jpg" class="img-fluid w-100"/>
<p>Below is the voltage data printed out in the serial monitor, the more intensive the light, the higher the number:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/dark-2.png" class="img-fluid w-100"/>
<p>Analog values displayed in the serial plotter:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/dark-3.png" class="img-fluid w-100"/>
<p>Below is the demo video of how the circuit worked:</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/DTRSwn83hMg" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>

<h5>Additional test - LDR color sensor</h5>
<p>I was so impressed by the power of the tiny LDR, hence, I would like to try another DIY circuit using it. Besides, one of the Final Project ideas that I abandoned was a <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-01-1-principles-and-practices.html#duolor">color detector</a>, therefore, I followed this <a href="https://www.instructables.com/id/DIY-Arduino-Color-Sensor/">tutorial</a> to wire an RGB color sensor circuit. The color sensor contained LEDs that illuminated the surface of the object. The LEDs blinked sequentially and the highest amount of light reflected back by the material surface while illuminated with the red, blue, green, and yellow LEDs was converted to voltage, then we can detect the color based on that. Detailed schematic diagram is described in the tutorial.</p>
<p>Below is the code from the tutorial. I modified it a bit and explained what I understand:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * pin variables
     */
    int ldr = 0;
    int b = 2;
    int r = 3;
    int g = 4;
    int y = 5;

    /* 
     * begin the serial communication and set all elements to OUTPUT
     */
    void setup()
    {
        pinMode(y, OUTPUT);
        pinMode(r, OUTPUT);
        pinMode(g, OUTPUT);
        pinMode(b, OUTPUT);
        pinMode(ldr, INPUT);
        Serial.begin(112500);
    }

    /* 
     * loop() function
     */
    void loop()
    {  
        allOff();
        delay(200);
        check();
    }

    /* 
     * check color of the object
     */
    void check()
    {
        delay(500);
        /* 
         * measure the voltage of the LDR from the beginning, with the immediate surroundings
         */
        double ambient = analogRead(ldr);

        /* 
         * measure the voltage of the LDR after blinking the green LED
         */
        digitalWrite(g, HIGH);
        delay(200);
        double green = analogRead(ldr);
        Serial.print("Green: ");
        Serial.println(green * 1.0);
        digitalWrite(g, LOW);
        delay(200);

        /* 
         * measure the voltage of the LDR after blinking the red LED
         */
        digitalWrite(r, HIGH);
        delay(200);
        double red = analogRead(ldr);
        Serial.print("Red: ");
        Serial.println(red * 1.0);
        digitalWrite(r, LOW);
        delay(200);

        /* 
         * measure the voltage of the LDR after blinking the blue LED
         */
        digitalWrite(b, HIGH);
        delay(200);
        double blue = analogRead(ldr);
        Serial.print("Blue: ");
        Serial.println(blue * 1.0);
        digitalWrite(b, LOW);
        delay(200);

        /* 
         * measure the voltage of the LDR after blinking the yellow LED
         * I don't really understand why but according to the tutorial 
         * I had to apply a factor of 1.17 to the yellow LED in order to get the correct result
         * maybe something with the forward voltage Vf?
         */
        digitalWrite(y, HIGH);
        delay(200);
        double yellow = analogRead(ldr);
        Serial.print("Yellow: ");
        Serial.println(yellow * 1.17);
        digitalWrite(y, LOW);
        delay(200);

        /* 
         * calculate the R,G,B,Y data
         * and detect the color = the piece of data with highest value
         */
        double rawData[] = {(green * 1.0), (red * 1.0), (blue * 1.0), (yellow * 1.17)};
        double maximum = ambient * 1.0;
        int color;

        for (int i = 0; i < 4; i++)  
        {
            if (maximum < rawData[i])
            {
                maximum = rawData[i];
                color = i+1;
            }
        }   

        Serial.println("");
        /* 
         * print the result to serial monitor
         */
        if (color == 0)
        {
            Serial.println("No object detected.");
            allOn();
            delay(1000);
            allOff();
        }
        else if (color == 1)
        {
            Serial.println("Green!");
            pulse(g);
        }
        else if (color == 2)
        {
            Serial.println("Red!");
            pulse(r);
        }
        else if (color == 3)
        {
            Serial.println("Blue!");
            pulse(b);
        }
        else if (color == 4)
        {
            Serial.println("Yellow!");
            pulse(y);
        }
        Serial.println("");
    }

    /* 
     * announce the detected color by blinking the related LED 
     */
    void pulse(int color)
    {
        for(int i = 0; i < 3; i++)
        {
            digitalWrite(color, HIGH);
            delay(1000);
            digitalWrite(color, LOW);
            delay(1000);
        }
    }

    void allOff()
    {
        digitalWrite(y, LOW);
        digitalWrite(r, LOW);
        digitalWrite(g, LOW);
        digitalWrite(b, LOW);
    }

    void allOn()
    {
        digitalWrite(y, HIGH);
        digitalWrite(r, HIGH);
        digitalWrite(g, HIGH);
        digitalWrite(b, HIGH);
    }
</code>
</pre>
<p>And here we have the wired circuit and testing objects! Ready to test!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/xSdAa0XtIZg" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>In fact, I couldn't use the small objects to test the program. I had to wrap my colorful sweaters around the whole breadboard in order to ensure better reflections and more accurate results. Otherwise, the sensor kept detecting the red color of my fingers.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/color.jpg" class="img-fluid w-100"/>
<p></p>

<h2 id="design">Designing and making a board with sensors</h2>
<p>This week seemed to be a good time for me to start thinking about how to design the electronics parts of my Final Project. I planned to use the <a href="https://www.espressif.com/sites/default/files/documentation/esp32-wroom-32_datasheet_en.pdf">ESP32 chip</a> as the central micro-controller that will manipulate all the input and output devices. I also bumped into this <a href="https://randomnerdtutorials.com/esp8266-multisensor-shield/">article</a> while studying the ESP32 and ESP8266 chips, and the multi-sensor shield seemed to be what I was looking for. Shields are modular circuit boards that piggyback onto the micro-controller board and instill it with extra functionalities. I found many Arduino shields while researching, and Raspberry Pi also has hardware with a similar concept called <a href="https://www.hifiberry.com/blog/what-is-a-raspberry-pi-hat/">HAT</a>. Shields usually have the same form-factor as the micro-controller board and it's important to make sure there is no overlapping usage of pins when stacking shields. I really fancied the idea of stacking shields to create a <strong>BIG MAC</strong> of circuits, so I went ahead to give it a try this week.</p>

<h5>Individual assignment - Design the ESP32 board and its multi-sensor shield</h5>
<h6 id="esp32">Designing the ESP32 board with KiCad</h6>
<p>I decided to build on top of the <a href="https://gitlab.fabcloud.org/barcelonaworkshops/barduino-2.0">Barduino 2.0</a> board designed by our instructor Eduardo. In order to understand the ESP32 chip's features and applications as well as how a wide range of sensors and actuators work, I went through this <a href="https://lastminuteengineers.com/esp32-arduino-ide-tutorial/">useful website</a>. Here they have a full ESP32 pinout diagram with detailed notes on what to do with the GPIO pins and some examples of using an ESP32 board with sensors. They also have a section for Arduino boards with more examples regarding motors, buzzers, and other specific modules. After digesting the content of the website, I came up with these below required building blocks for my board:</p>
<ul>
<li>The board will have basic blocks that are exactly the same with ones from the original Barduino board, namely: the programming switch and the reset button (to flash programs to the chip), a user-programmable LED (for debugging), an FTDI area including the FTDI header and a 3.3V LDO regulator (to convert higher voltage from USB and the battery to a 3.3V voltage that is not damaging to the ESP32 chip).</li>
<li>The board will be powered by a battery when being packaged into my <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/final-project.html">dragon</a>. I was not so sure about available jacks at Fab Lab Barcelona, but Josep recommended LiPo batteries which usually go together with JST plugs. In addition, I was thinking about using these <a href="https://www.alibaba.com/product-detail/High-quality-2pin-3pin-4pin-5pin_60709244727.html?spm=a2700.details.deiletai6.4.7adb7decjQbTy5">magnetic pogo pins</a> or these <a href="https://www.digikey.es/es/datasheets/bournsinc/bourns-inc-70adm">male</a> and <a href="https://www.bourns.com/docs/Product-Datasheets/70ADF.pdf">female</a> connectors to connect the boards electrically, with or without transferring data I didn't know. Therefore I also added extra I2C and power headers, just in case.</li>
<li>It seems like there are only +6V LiPo batteries available, that's why I added a 5V LDO regulator to the circuit. After reading the <a href="https://www.diodes.com/assets/Datasheets/ZLDO1117.pdf">data sheet of the ZLDO1117 regulators</a> (which is used in the Barduino board), I noticed some points: a suitable range of input voltage should be <strong>6.4V < Vin < 12V</strong>, both of them have SOT223 SMD package available, a minimum of 4.7µF capacitor must be connected from Vout to GND to ensure stability, and all ZLDO1117-X regulators will be wired in the same way. I also referred to this <a href="http://electronics-diy.com/3.3v-and-5v-power-supply.php">tutorial</a> to wire a circuit that allows 3.3V and 5V output from a +6V power supply. About capacitors, I followed exactly what Eduardo did, because I trust him.</li>
<li>For the input sensors, I'm going to use an HC-SR04 ultrasonic sensor and an LDR. Both require 5V to operate. For the output parts, I'm thinking about having 2 <a href="https://components101.com/sites/default/files/component_datasheet/SG90%20Servo%20Motor%20Datasheet.pdf">SG90 micro servo motors</a>, an <a href="https://cdn-shop.adafruit.com/datasheets/WS2812B.pdf">addressable RGB LED strip</a>, and an <a href="https://components101.com/buzzer-pinout-working-datasheet#:~:text=Active%20Passive%20Buzzer&text=It%20is%20very%20small%20and,component%20in%20most%20electronic%20applications.&text=This%20buzzer%20can%20be%20used,ranging%20from%204V%20to%209V.">active buzzer</a>. As far as I know, they work fine at 5V. Hence, I assumed the best strategy could be powering both the ESP32 boards and the sensor/actuator shields at around 5V, and then perform some level shiftings to input the proper voltages to some specific elements. In order to place everything except for the motors on top, I put the LED strip adapter and the buzzer in the same shield with the sensors.</li>
<li>I took a look at this <a href="https://hackaday.com/2016/12/05/taking-it-to-another-level-making-3-3v-and-5v-logic-communicate-with-level-shifters/">link</a> in order to understand different ways of level shifting. In addition to the voltage regulator, I applied the voltage divider method to convert 5V signals from the sensors to 3.3V signals that are safe for the chip. The formula I was using is <strong>Vin/Vout = (R1 + R2)/R2</strong>. The HC-SR04 sensor can be trigger with a 3.3V signal, so that part would be fine.</li>
<li>I also followed the solution from the above-mentioned link and used N-MOSFET transistors to amplify the 3.3V outputs of the chip to higher ones which the output devices might require. I was not so certain about the required voltage of the output devices yet or whether I would need loads/resistors with a specific resistance, therefore here I separated <a href="https://www.youtube.com/watch?v=vudSUDFWoQE">an example N-MOSFET circuit</a> into 2 parts: the lower voltage side will be handled by the ESP32 board, and the higher voltage side will be handled by the motor shield. Little did I know that I would face problems using the MOSFETs soon after, but that would be discussed better in the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-11-output-devices.html#esp32">11th week</a>.</li>
</ul>
<p>Below is the first version of my KiCad schema:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/kicad-1.png" class="img-fluid w-100"/>
<p>We then had an electronics debugging session with the instructors, and I made some adjustments after we discussed my PCB layout:</p>
<ul>
<li>I decided to include a <a href="https://components101.com/dht11-temperature-sensor">DHT11 temperature and humidity sensor</a> to my design.</li>
<li>I also added some pull-up resistors in order to provide a default state for the I2C signal lines.</li>
</ul>
<p>After getting back to the lab after the lockdown, I had a discussion with our instructor Eduardo and we realized that I should power the motor shield first and from that shield, power will be passed to the ESP32 board, else the current passed from the ESP32 board to the shields will be constrained to 1 Amp. So, there you go the final schema of the board:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/kicad-4.png" class="img-fluid w-100"/>
<p>I tried to avoid placing elements and traces near the antenna area, and added some 0Ω resistors here as soldering jumpers while routing the PCB layout:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/kicad-5.png" class="img-fluid w-100"/>
<p>I also exported the board to <code>.step</code> in KiCad and then <code>.fbx</code> in Fusion 360 to see how it will look like after being stuffed. The reason why I spent time doing this extra thing was to imagine more clearly how I should stack the shields to the board as well as where I should plug external elements. It also helped me to estimate how big the 3D-printed housing parts (the dragon's head) should be (at the moment, something that can contain approximately <strong>95 x 85 x 65mm</strong> electronic parts).</p>
<p></p>
<div class="sketchfab-embed-wrapper text-center">
    <iframe title="A 3D model" width="640" height="480" src="https://sketchfab.com/models/17f45719e11e44d09f237627a74268be/embed?autostart=1&amp;ui_controls=1&amp;ui_infos=1&amp;ui_inspector=1&amp;ui_stop=1&amp;ui_watermark=1&amp;ui_watermark_link=1" frameborder="0" allow="autoplay; fullscreen; vr" mozallowfullscreen="true" webkitallowfullscreen="true"></iframe>
</div>  
<p></p>
<p>The layout seemed to be good, or at least I was kinda happy with it at the moment. The next step was to prepare the traces for the drilled holes. Also, the copper under the antenna area of the ESP32 chip is required to be removed (due to the <a href="https://en.wikipedia.org/wiki/Faraday_cage">Faraday cage</a> effect, as Eduardo explained). That's why I had 3 layers exported.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/kicad-6.png" class="img-fluid w-100"/>
<p>Finally, I imported the 3 <code>.png</code> files to Fab Modules in order to check everything again. There were some parts where the traces were a bit close to each other, but they still looked manageable since they were completely separated. Let's see, finger-crossed!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/cam-1.png" class="img-fluid w-100"/>
<p></p>

<h6>Designing the multi-sensor shield with KiCad</h6>
<p>The sensor shield was indeed designed during July when we got back to the lab after the lockdown, and it was the last PCB I designed. To be honest, I was kinda tired from routing all the PCBs, so I used <strong>A LOT OF</strong> jumpers while designing it. The only thing I should pay attention to was to place the sensors and actuators in a way that will make it easier for wiring and assembling later. I have here 2 headers for the ultrasonic sensor (which will be placed at the nose of the dragon) and the LED strip (which will be placed at the teeth).</p>
<p>Below we have the KiCad schema and the PCB layout of the sensor shield:</p>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/ss-1.png" class="img-fluid w-100"/>
<p></p>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/ss-2.png" class="img-fluid w-100"/>
<p></p>
<p>And below we have the fancy 3D thing. I figured out that we can export the 3D PCB layout to <code>.step</code>, and then continue to work on it with CAD software (Blender or Fusion 360). We also can export the layout to a <code>.fbx</code> model which includes all textures and materials. That is cool!</p>
<div class="sketchfab-embed-wrapper text-center">
    <iframe title="A 3D model" width="640" height="480" src="https://sketchfab.com/models/178da3f50d7540f8b747dc9a70ea4fb5/embed?autostart=1&amp;ui_controls=1&amp;ui_infos=1&amp;ui_inspector=1&amp;ui_stop=1&amp;ui_watermark=1&amp;ui_watermark_link=1" frameborder="0" allow="autoplay; fullscreen; vr" mozallowfullscreen="true" webkitallowfullscreen="true"></iframe>
</div>
<p></p>
<p>I didn't notice a small problem here with the toolpath: some of the DHT11 traces were too close to each other. This actually led to a little accidental overheating which will be described <a href="#ldo">below</a>.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/cam-2.png" class="img-fluid w-100"/>
<p></p>

<h6>Milling and soldering the boards</h6>
<p>Not much to say about the milling and soldering process:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/mill-1.jpg" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/mill-2.jpg" class="img-fluid w-100"/>
<p id="ldo">Then I had a problem trying to power the PCBs. The 3.3V voltage regulator was overheated too fast when I plugged the sensor shield onto the ESP32 board, and the temperature was too high that I couldn't even touch it. After checking again the ESP32 board with the help from our instructor Josep, we figured out that GND and 3.3V were connected right under the DHT11. This was a bad mistake for not carefully checking the milled board with the multimeter. To fix that, we used a knife to separate the traces.</p>  
<p></p>

<h6>Programming the board and reading sensors</h6>
<p>Finally, they worked totally fine!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/wire.png" class="img-fluid w-100"/>
<p></p>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/sensor.png" class="img-fluid w-100"/>
<p>I used the <i>NewPing</i> library for reading the ultrasonic sensor by creating a <strong>sonar</strong> object. Below is the working code:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    #include &lt;DHT.h&gt;
    #include &lt;NewPing.h&gt;
    #define BUILTIN_LED 13 
    #define TRIGGER_PIN 32
    #define ECHO_PIN 35
    #define DHT_PIN 33
    #define LDR_PIN 34

    /* 
     * variables
     */
    int ldrValue;
    float tempValue;
    float humValue;
    unsigned int uS; 
    float distance;

    /* 
     * NewPing and DHT libraries
     */
    NewPing sonar(TRIGGER_PIN, ECHO_PIN, 200);
    #define DHTTYPE DHT11 
    DHT dht(DHT_PIN, DHTTYPE);
    
    void setup() {
        pinMode(BUILTIN_LED, OUTPUT);
        Serial.begin(115200);
        dht.begin();
    }
    
    void loop() {
        delay(2000);
        ldrValue = analogRead(LDR_PIN); 
        humValue = dht.readHumidity();
        tempValue = dht.readTemperature();
        uS = sonar.ping();
        distance = uS / US_ROUNDTRIP_CM;

        /* 
         * print data
         */
        Serial.print("Distance: ");
        Serial.print(distance); 
        Serial.print("cm");
        Serial.print(" \n"); 
        Serial.print("LDR: ");
        Serial.print(ldrValue); 
        Serial.print(" \n"); 
        Serial.print(F("Humidity: "));
        Serial.print(humValue);
        Serial.print(" \n"); 
        Serial.print(F("Temperature: "));
        Serial.print(tempValue);
        Serial.print("°C");
        Serial.println(" \n");
        
        /* 
         * debugging
         */
        digitalWrite(BUILTIN_LED, LOW);   
        delay(1000);                     
        digitalWrite(BUILTIN_LED, HIGH);  
        delay(1000); 
    }
</code>
</pre>
<p></p>

<h2>Conclusion</h2>
<p>It was really fun playing around with these lovely sensors. Our instructor Oscar gave us an interesting class about <a href="http://fabacademy.org/2020/labs/barcelona/local/#material/extras/week09/dspbasics">digital signal processing</a>, and I'd love to try them out in the upcoming assignments in order to take a more rigid and mathematical approach on processing and visualizing data.</p>
<p></p>

<div class="container w-100 text-center py-4">
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-08-embedded-programming.html">Previous Assignment</a>
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-10-applications-and-implications.html">Next Assignment</a>
</div>