---
layout: assignment
title: Output Devices
thumb: 12.png
assignment: add an output device to a designed micro-controller board and program it to do something (individual); to measure the power consumption of an output device (in-group). 
active: 1
---

<p class="font-italic font-weight-bold">* All C++ and KiCad files can be downloaded <a href="https://gitlab.fabcloud.org/academany/fabacademy/2020/labs/barcelona/students/tue-ngo/tree/master/assets/img/output">here</a>.</p>

<p class="font-italic font-weight-bold">* This week's <a href="#group">group assignment</a> is to measure the power consumption of an output device.</p>

<p class="font-italic font-weight-bold">* My plan for this week's progress was to make and/or test <a href="#diy">basic actuators</a> in order to understand how they work and to <a href="#design">design a board with built-in actuators</a>. The board will be fabricated after the lockdown.</p>

<h2>Characterizing output devices</h2>
<p>I was really excited this week because I'd like to learn about all types of motors. Motors are electromagnetic actuators that use an induced magnetic field to make some sort of actuation. First, let me put here a rundown on the differences between DC, servo, and stepper motors:</p>
<ul>
<li>DC (Direct Current) motors are 2-wire (power & ground), continuous rotation motors that run at a high <a href="https://en.wikipedia.org/wiki/Revolutions_per_minute#:~:text=Revolutions%20per%20minute%20(abbreviated%20rpm,rotation%20around%20a%20fixed%20axis.">RPM (revolutions per minute)</a>. That's why they are used for anything that needs to spin at a high rotational speed, for example, cooling fans or car wheels. The speed of DC motors is controlled using <a href="https://en.wikipedia.org/wiki/Pulse-width_modulation#:~:text=Pulse%2Dwidth%20modulation%20(PWM),it%20up%20into%20discrete%20parts.">PWM</a>. An H-bridge is a circuit configuration commonly used to control the speed and direction of a DC motor, thus allows it to move forward or backward.</li>
<li>Servo motors are generally an assembly of 4 things: a DC motor, a gearing set, a control circuit, and a position sensor (usually a potentiometer). Servo motors receive a control signal that represents an output position and applies power to the DC motor until the shaft turns to the correct position determined by the potentiometer. The angle of rotation is limited to 180 degrees. The position of servo motors can be controlled more precisely than those of standard DC motors, and that's why servo motors are used for specific tasks where we need fast, high torque, accurate rotation within a limited angle, such as moving a robotic arm within a certain range.</li>
<li>A stepper motor is a servo motor that uses a different method of motorization. Where a servo motor uses a continuous rotation DC motor and integrated controller circuit, stepper motors utilize multiple toothed electromagnets arranged around a central gear to define the position. Stepper motors require an external micro-controller to individually energize each electromagnet and make the motor shaft turn. Each rotation from one electromagnet to the next is called a "step", and thus the motor can be turned at full 360-degree rotation. They are suited for 3D printers and similar devices where the position is fundamental.</li>
</ul>
<h5 id="group">Group assignment - Probe the SG90 micro servo motor</h5>
<p>I knew that I'd use DC motors for my Final Project, and I'd mostly work with stepper motors for the machine design week. So I decided to work with the servo motor for this group assignment. We were required to measure the power consumption of an actuator. The power consumption measured in Watt and the current consumption measured in Ampere are often used to determine how much energy is actually drained from the battery to complete a specific task over a specific time. In an attempt to measure the power consumption of the SG90 micro servo which I got from a starter Arduino kit, I used an Arduino Uno board as the power supply as well as the MCU. Below is the simple code I wrote to make my servo rotate at a random degree:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * Arduino library for controlling hobby servo motors
     */ 
    #include &lt;Servo.h&gt;
    
    /* 
     * get a random degree within the angle limitation 
     * and declare a Servo object called 'servo'
     */ 
    int pos = random(0,180);
    Servo servo;

    /* 
     * indicate the Arduino pin 9 connected to the signal pin of the micro servo  
     */
    void setup()
    {
        servo.attach(9);
    }

    /* 
     * rotate the micro servo  
     */
    void loop()
    {
        servo.write(pos);
        delay(50);
        pos = random(0,180);
        delay(250);
    }
</code>
</pre>
<p>This video recorded the process when I switched from 5V power supply to 3.3V (0:04), then from 3.3V to 5V (0:12). It was obvious that even though the micro servo can operate at 3.3V, a higher operating voltage will make it rotate happily at a higher speed. I also tried to put some force to the servo arm using my fingers, and at 5V I had to apply a slightly higher force to make it stop rotating.</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/V542tkYpPZ4" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>Having some issues while measuring current consumption with the multimeter, I reached out to my classmate <a href="http://fabacademy.org/2020/labs/barcelona/students/david-prieto/assignments/week12/">David Prieto</a>, and since he was working on the SM-S2309S micro servo with similar technical parameters, we worked online together to find solutions for problems. Finally, I could do a real "group" assignment during the lockdown. David documented well the whole process, but here I described my own mistakes and learning. While I could measure voltage by poking at VCC and GND in parallel, I had to find a way to measure current in series. I attempted to physically interrupt the flow of current between Arduino Uno and the micro servo, and put the multimeter into the circuit:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/group.jpg" class="img-fluid w-100"/>
<p>The way I wired the circuit before talking to David (the picture on the left) was totally wrong because I basically still put the multimeter in parallel. To fix that, I unplugged the jumper wires from the breadboard and instead taped them down, then I could poke the probes of the multimeter in-line. However, I could only measure a small ~3mA when my servo stayed still in its idle state. It was too tricky for me to ensure a stable connection between the jumper wires and the probes since I didn't have a soldering iron with me. Hence, I will include here the result from David. The measured current varied from ~150mA to almost ~200mA, which means the power consumption of the servo ranged from <strong>0.75 - 0.9W</strong> with no load attached, according to the formula <strong>W = V x A</strong>:</p>
<div class="text-center">
<iframe width="640" height="360" src="https://www.youtube.com/embed/5Yp6Rjr0V2E" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>Since my 4$ multimeter can only measure a maximum current of 200mA, I couldn't do anything further to measure the power consumption when there are loads attached. Fortunately, my classmate <a href="http://fabacademy.org/2020/labs/barcelona/students/benjamin-scott/week-12.html">Benjamin Scott</a> also performed some measurements with the same SG90 servo, and his result was that the servo can consume up to <strong>5V x 580mA ~ 2.9W</strong> when there are torque loads.</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/D6WeG2l2d2U" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>That's it for the group assignment. More or less I understood some characteristics of the SG90 servo. I will try to do this group assignment properly by using an oscilloscope when we get back to the lab.</p>
<p></p>


<h2 id="diy">Making and testing basic actuators</h2>
<h5>Additional test - RGB LED</h5>
<p>I continued to try an electroluminescence actuator: a common cathode RGB LED to which we give PWM signals to the anode of the internal red, green and blue LEDs. As per Santi's suggestion, I want to use these RGB LEDs in my Final Project as a way to help kids debug their trials: red means "Error!", green means "Success!", and yellow means "Warning!" (while there are weird conditions or behaviors detected). The wiring of the circuit was really basic: I connected the cathode of the RGB LED (the longest leg) to the GND pin of Arduino Uno and wired the other three pins to pin 9, 10, 11 through 220Ω resistors. Pin 9 was connected to the internal blue LED, pin 10 to the green one, and pin 11 to the red one. They are all <a href="https://www.arduino.cc/en/tutorial/PWM">PWM</a> pins which can be used to give a desired analog output by varying the width of the pulse.</p>
<p>Below is the easy code I wrote to sequentially display RGB and CYMK primary colors:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * pin variables 
     */ 
    int r = 11;
    int g = 10;
    int b = 9;

    void setup() {
        pinMode(r, OUTPUT);
        pinMode(g, OUTPUT);
        pinMode(b, OUTPUT);
    }

    /* 
     * display certain colors based on the R,G,B values of that color
     */ 
    void loop() {
        mix(255, 0, 0); // red
        delay(1000);
        mix(0, 255, 0); // green
        delay(1000);
        mix(0, 0, 255); // blue
        delay(1000);
        mix(255, 255, 255); // white
        delay(1000);
        mix(0, 255, 255); // cyan
        delay(1000);
        mix(255, 0, 255); // magenta
        delay(1000);
        mix(255, 255, 0); // yellow
        delay(1000);
        mix(255, 255, 255); // white
        delay(1000);
    }

    /* 
     * use analogWrite() to change the intensity of 3 internal LEDs, with analog values from 0 - 255
     */ 
    void mix(int red, int green, int blue)
    {
        analogWrite(r, red);
        analogWrite(g, green);
        analogWrite(b, blue);
    }
</code>
</pre>
<p>The R,G,B values of a certain color can be checked <a href="https://imagecolorpicker.com/en/">here</a>. Look at this, what a lovely thing!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/EKHnPFjP7tI" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<h5>Additional test - Active and passive buzzers</h5>
<p>My next experiment was to try piezoelectric buzzers. There are 2 types of buzzer: an active buzzer that has additional circuitry in it which makes it simpler to control (turn on/off) but limits the buzzer to only one tone; and a passive buzzer which can make different tones, but the MCU has to provide an electronic signal at a desired frequency and that frequency will determine the tone. The connection was easy: (-) pin of the buzzer was connected to GND pin of Arduino Uno, and digital pin 4 of Arduino Uno was connected to the (+) pin of the buzzer.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/buzzer-1.jpg" class="img-fluid w-100"/>
<p>In order to test the active buzzer as an output controlled by an input, I used the HR-SC04 sensor. If the distance measured is less than 5cm, the buzzer will make a sharp alarm tone. This could be developed to a custom behavior of my dragon: whenever someone gets close to it, it will roar.</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/grTK2QwZSp4" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p id="mario">I tried to play some music with the passive buzzer, and since I'm a big fan of Super Mario, I looked for this online <a href="https://www.hackster.io/jrance/super-mario-theme-song-w-piezo-buzzer-and-arduino-1cc2e4">tutorial</a>, mostly for the notes included in the theme song. I also wrote the functions myself and simplified the program a bit. A cheatsheet for converting notes to frequency values can be checked <a href="https://artificialtunes.tumblr.com/post/8249542526/waves-in-space-part-1">here.</a></p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/yLeUV1K9MD8" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>Below are the programs I used for both buzzers:</p>
<ul class="nav nav-tabs nav-justified d-flex">
  <li class="flex-fill p-2 mr-2 my-2"><a data-toggle="tab" href="#active">Active buzzer - HR-SC04 sensor</a></li>
  <li class="flex-fill p-2 mr-2 my-2"><a data-toggle="tab" href="#passive">Passive buzzer - Super Mario</a></li>
</ul>
<div class="tab-content">
    <pre id="active" class="tab-pane active bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
    <code>
    /* 
     * pin variables
     */
    int trig = 11;
    int echo = 12;
    int buzzer = 4;

    /* 
     * useful ultrasonic sensor's variables
     */
    long unsigned duration = 0;
    int distance = 0;
    const int threshold = 5;

    void setup() {
        pinMode(trig, OUTPUT); 
        pinMode(echo, INPUT);
        pinMode(buzzer, OUTPUT);
    }

    /* 
     * turn buzzer on/off if the distance is less than 5cm 
     */
    void loop() {
        digitalWrite(buzzer, LOW);  

        digitalWrite(trig, LOW);
        delay(50);
        digitalWrite(trig, HIGH);
        delay(100);
        digitalWrite(trig, LOW);

        duration = pulseIn(echo, HIGH);
        distance = (duration * 0.034)/2;

        if (distance < threshold) {
            digitalWrite(buzzer, HIGH);   
            delay(1000);                       
            digitalWrite(buzzer, LOW);    
            delay(1000);
        }   

        delay(200);                       
    }
    </code>
    </pre>
    <pre id="passive" class="tab-pane bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
    <code>
    /* 
     * all frequency values of all notes here in case I want to try other 80s theme songs 
     */
    #define NOTE_B0  31
    #define NOTE_C1  33
    #define NOTE_CS1 35
    #define NOTE_D1  37
    #define NOTE_DS1 39
    #define NOTE_E1  41
    #define NOTE_F1  44
    #define NOTE_FS1 46
    #define NOTE_G1  49
    #define NOTE_GS1 52
    #define NOTE_A1  55
    #define NOTE_AS1 58
    #define NOTE_B1  62
    #define NOTE_C2  65
    #define NOTE_CS2 69
    #define NOTE_D2  73
    #define NOTE_DS2 78
    #define NOTE_E2  82
    #define NOTE_F2  87
    #define NOTE_FS2 93
    #define NOTE_G2  98
    #define NOTE_GS2 104
    #define NOTE_A2  110
    #define NOTE_AS2 117
    #define NOTE_B2  123
    #define NOTE_C3  131
    #define NOTE_CS3 139
    #define NOTE_D3  147
    #define NOTE_DS3 156
    #define NOTE_E3  165
    #define NOTE_F3  175
    #define NOTE_FS3 185
    #define NOTE_G3  196
    #define NOTE_GS3 208
    #define NOTE_A3  220
    #define NOTE_AS3 233
    #define NOTE_B3  247
    #define NOTE_C4  262
    #define NOTE_CS4 277
    #define NOTE_D4  294
    #define NOTE_DS4 311
    #define NOTE_E4  330
    #define NOTE_F4  349
    #define NOTE_FS4 370
    #define NOTE_G4  392
    #define NOTE_GS4 415
    #define NOTE_A4  440
    #define NOTE_AS4 466
    #define NOTE_B4  494
    #define NOTE_C5  523
    #define NOTE_CS5 554
    #define NOTE_D5  587
    #define NOTE_DS5 622
    #define NOTE_E5  659
    #define NOTE_F5  698
    #define NOTE_FS5 740
    #define NOTE_G5  784
    #define NOTE_GS5 831
    #define NOTE_A5  880
    #define NOTE_AS5 932
    #define NOTE_B5  988
    #define NOTE_C6  1047
    #define NOTE_CS6 1109
    #define NOTE_D6  1175
    #define NOTE_DS6 1245
    #define NOTE_E6  1319
    #define NOTE_F6  1397
    #define NOTE_FS6 1480
    #define NOTE_G6  1568
    #define NOTE_GS6 1661
    #define NOTE_A6  1760
    #define NOTE_AS6 1865
    #define NOTE_B6  1976
    #define NOTE_C7  2093
    #define NOTE_CS7 2217
    #define NOTE_D7  2349
    #define NOTE_DS7 2489
    #define NOTE_E7  2637
    #define NOTE_F7  2794
    #define NOTE_FS7 2960
    #define NOTE_G7  3136
    #define NOTE_GS7 3322
    #define NOTE_A7  3520
    #define NOTE_AS7 3729
    #define NOTE_B7  3951
    #define NOTE_C8  4186
    #define NOTE_CS8 4435
    #define NOTE_D8  4699
    #define NOTE_DS8 4978

    /* 
     * pin variables
     */
    int buzzer = A0;

    /* 
     * list down all notes of the melody
     */
    int melody[] = {
    NOTE_E7, NOTE_E7, 0, NOTE_E7, 0, NOTE_C7, NOTE_E7, 0, NOTE_G7, 0, 0, 0, NOTE_G6, 0, 0, 0,
    NOTE_C7, 0, 0, NOTE_G6, 0, 0, NOTE_E6, 0, 0, NOTE_A6, 0, NOTE_B6, 0, NOTE_AS6, NOTE_A6, 0
    };
    
    void setup()
    {
        pinMode(buzzer, OUTPUT);
    }
    
    void loop()
    {
        /* 
         * loop through all notes of the melody
         */    
        for (int i = 0; i < (sizeof(melody) / sizeof(int)); i++) {   

            // tone(pin, frequency, duration) 
            // to calculate the note duration, take one second divided by the note type
            // e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
            tone(buzzer, melody[i], 125); //1000 / 8

            // to distinguish the notes, set a minimum time between them
            delay(150); //125 * 1.2

            // stop the tone playing
            tone(buzzer, 0, 125);
        }
    }
    </code>
    </pre>
</div>
<p></p>

<h2 id="design">Designing and making a board with actuators</h2>
<h5>Individual assignment - Design the ESP32 board and its motor shield</h5>
<p>This week, I continued to design the shield which will act as a bridge between my main ESP32 board and the motors.</p>
<h6 id="esp32">Designing the ESP32 board with KiCad</h6>
<p>The ESP32 board was designed in the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-09-input-devices.html#esp32">9th week</a>, but I mostly talked about how I came up with the input parts there. Here I would like to continue discussing the output parts:</p>
<ul>
<li>For the output parts, I was thinking about connecting the board to 2 <a href="https://es.banggood.com/DC-3V-6V-DC-1120-Gear-Motor-TT-Motor-for-Arduino-Smart-Car-Robot-DIY-p-1260117.html?gpla=1&gmcCountry=ES&currency=EUR&createTmp=1&utm_source=googleshopping&utm_medium=cpc_bgs&utm_content=lijing&utm_campaign=pla-esg-all-pc-0319&ad_id=426521747247&gclid=CjwKCAjw4871BRAjEiwAbxXi28C1Bu_1sppbIQ6ozmruStsq2tYwkTjwgpfLaRW_0QRuKRM4p5pZZxoCLbQQAvD_BwE&cur_warehouse=CN">DC motors</a> through a motor shield which will piggyback onto the ESP32 board. A small DC motor usually has a stall torque of 0.85 kgf·cm. Stall torque is the torque load that can be applied on the device to make it stop rotating. The value of 0.85 kgf·cm means the amount torque we would get if we were to bolt a 1cm long arm to the shaft horizontally and hang a 0.85kg mass at the end of the arm. Fortunately, my Final Project doesn't require a long arm at all and it also won't weight heavier than 0.85kg x 2 = 1.7kg, therefore it might not need a stronger (and more dangerous) motor. 2 DC motors should be fine.</li> 
<li>For the power supply, I will go with a +1A LiPo battery, since it will be sufficient for an ESP32 MCU in active mode (~250mA), 2 DC motors that require up to 1A continuous consumption each, and other components. The motor shield will be powered first and from the shield, power will be passed to the ESP32 board and the DC motors, because if I power the ESP32 first, the current passed from the ESP32 board to the shields and the motors will be constrained to 1 Amp.</li>
<li>At first, I wanted to use N-MOSFET transistors as switches to turn on 2 motors: 1 CW motor (to move forward) and 1 CCW motor (to move backward). MOSFET is basically a transistor which can cut high currents or voltages but handled by a microcontroller (interesting tutorial <a href="https://www.youtube.com/watch?v=vudSUDFWoQE">here</a>). The larger the voltage we apply to the GATE, ideally, the lower the resistance, meaning the MOSFET is ON. I followed one of the solutions from this <a href="https://hackaday.com/2016/12/05/taking-it-to-another-level-making-3-3v-and-5v-logic-communicate-with-level-shifters/">link</a>. Little did I know that many MOSFETs ask for a minimum of 5V to be able to open a circuit. Oscar advised me to use other transistors instead, or to use commercial boards with TXS010X chips, or to look for MOSFETs with low activation voltage that can be driven by 3.3V logic. I was convinced tremendously by this <a href="https://www.raspberrypi.org/forums/viewtopic.php?t=101391">thread</a> that <a href="https://www.diodes.com/assets/Datasheets/ds30144.pdf">BSS138</a>, <a href="https://www.onsemi.com/pub/Collateral/2N7000-D.PDF">2N7000</a>, and <a href="https://www.onsemi.com/pub/Collateral/FDN337N-D.PDF">FDN337N</a> work fine for most applications. They have SOT23 SMD packages available and their Vgs(th) ranges are quite low (0.5 - 1.5V for the BSS138, 0.8 - 3V for the 2N7000, and 0.4 - 1V for the FDN337N). Vgs is the voltage from gate to source and Vgs(th) is the voltage at which the MOSFET will begin to conduct or to be turned on partially. The FDN337N's datasheet also clearly states in the Rds(on) section that Vgs can go down to 2.5V TTL level.</li>
<li>After researching online for a while, I changed my mind and decided to control the motors with H-bridge circuits in order to make the motors rotate differently. My first idea was to use a commercial <a href="https://www.youtube.com/watch?v=dyjo_ggEtVU">L298N</a> motor driver, as it can handle 2 DC motors at the same time. However, it requires too many inputs (2 data inputs and 1 enabler per motor), I couldn't find available KiCad footprint, and I also want to limit the number of commercial elements used in my design. Therefore, I decided to use 2 A4953 chips to handle 2 DC motors since I saw the footprint in the KiCad library provided by the instructors. As per the <a href="https://www.digikey.es/en/datasheets/allegro-microsystems-llca49523datasheetashx">data sheet</a>, they seem to work fine with 3.3V logic inputs.</li>
<li>I put other output devices such as active buzzer and RGB LED strip on the upper shield, together with the sensors. They work fine at 5V, so they won't need any external power supply or logic level shifting.</li>
</ul>
<p></p>
 <img src="{{site.baseurl}}/assets/img/assignments/week-09/kicad-4.png" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-09/kicad-5.png" class="img-fluid w-100"/>
<p></p>

<h6>Designing the motor shield with KiCad</h6>
<p>I took a look at <a href="http://fabacademy.org/archives/2015/sa/students/siu.gonzalo/index.html">this documentation</a> to design my motor shield. But instead of using 2x2 pin headers to wire the motors onto the shield, I used a 3.5mm terminal blocks. The shield was designed during July when we got back to the lab after the lockdown, and to be honest, I was kinda tired from routing all the PCBs, so I used <strong>A LOT OF</strong> jumpers.</p>
<p>Below we have the KiCad schema and the PCB layout of the motor shield:</p>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/kicad.png" class="img-fluid w-100"/>
<p></p>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/pcb-motor.png" class="img-fluid w-100"/>
<p></p>
<p>And below we have the fancy 3D thing:</p>
<div class="sketchfab-embed-wrapper text-center">
    <iframe title="A 3D model" width="640" height="480" src="https://sketchfab.com/models/f7074185036f423f976e6d241ab4d3b1/embed?autostart=1&amp;ui_controls=1&amp;ui_infos=1&amp;ui_inspector=1&amp;ui_stop=1&amp;ui_watermark=1&amp;ui_watermark_link=1" frameborder="0" allow="autoplay; fullscreen; vr" mozallowfullscreen="true" webkitallowfullscreen="true"></iframe>
</div>
<p></p>

<h6>Milling and soldering the boards</h6>
<p>Not much to say about the milling and soldering process:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/cam-motor-1.png" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/cam-motor-2.png" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/mill-1.png" class="img-fluid w-100"/>
<p>Here I have a little problem while trying to power the motor shield. I placed the footprint of the JST battery connector in the wrong direction, hence I couldn't plug the jack onto it. To avoid re-designing the shield, I cut, swap, and re-solder the wires of the battery as the image below:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-11/mill-2.png" class="img-fluid w-100"/>
<p></p>
<p>Finally, it's time to assemble the shields. I was super satisfied to see this <i>BIG MAC</i> of PCBs. Om nom nom!</p>
<div class="text-center">
<iframe width="640" height="360" src="https://www.youtube.com/embed/_krMG-EDa98" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>

<h6>Programming the board and driving actuators</h6>
<p>First, I tested the buzzer and an LED strip of 3 pixels with the below code. There are basic NeoPixel examples that can be found everywhere online. However, I still put some comments here:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * libraries and pin variables
     */ 
    #include &lt;Adafruit_NeoPixel.h&gt;
    #define BUILTIN_LED 13 
    #define BUZZ_PIN 14
    #define RGB_PIN 26

    // number of pixels in NeoPixel strip
    #define LED_COUNT 3
    // declare NeoPixel strip object
    Adafruit_NeoPixel strip(LED_COUNT, RGB_PIN, NEO_GRB + NEO_KHZ800);
    
    /* 
     * initialize NeoPixel strip object
     */
    void setup() {
        pinMode(BUILTIN_LED, OUTPUT); 
        pinMode(BUZZ_PIN, OUTPUT);
        strip.begin();           
        strip.show();            
        strip.setBrightness(50); 
    }
    
    void loop() {
        digitalWrite(BUZZ_PIN, HIGH);   
        delay(1000);                       
        digitalWrite(BUZZ_PIN, LOW);    
        delay(1000);

        colorWipe(strip.Color(255, 0, 0), 50); // red
        colorWipe(strip.Color(0, 255, 0), 50); // green
        colorWipe(strip.Color(0, 0, 255), 50); // blue

        digitalWrite(BUZZ_PIN, HIGH);   
        delay(1000);                       
        digitalWrite(BUZZ_PIN, LOW);    
        delay(1000);

        rainbow(10);

        digitalWrite(BUZZ_PIN, HIGH);   
        delay(1000);                       
        digitalWrite(BUZZ_PIN, LOW);    
        delay(1000);

        theaterChase(strip.Color(127, 127, 127), 50); //white, theater chase effect

        digitalWrite(BUILTIN_LED, LOW);   
        delay(1000);                     
        digitalWrite(BUILTIN_LED, HIGH);  
        delay(1000);  
    }

    void colorWipe(uint32_t color, int wait) {
        for(int i = 0; i < strip.numPixels(); i++) { 
            strip.setPixelColor(i, color);         
            strip.show();                          
            delay(wait);                          
        }
    }

    void theaterChase(uint32_t color, int wait) {
        for(int a = 0; a < 10; a++) {  
            for(int b = 0; b < 3; b++) { 
            strip.clear();         
            for(int c = b; c < strip.numPixels(); c += 3) {
                strip.setPixelColor(c, color); 
            }
            strip.show(); 
            delay(wait);  
            }
        }
    }

    void rainbow(int wait) {
        for(long firstPixelHue = 0; firstPixelHue < 5*65536; firstPixelHue += 256) {
            for(int i = 0; i < strip.numPixels(); i++) { 
                int pixelHue = firstPixelHue + (i * 65536L / strip.numPixels());
                strip.setPixelColor(i, strip.gamma32(strip.ColorHSV(pixelHue)));
            }
            strip.show(); 
            delay(wait);  
        }
    }
</code>
</pre>
<p>They seem to work totally fine. After designing the dragon's head, I can have more data lines and try many more effects.</p>
<div class="text-center">
<iframe width="640" height="360" src="https://www.youtube.com/embed/4Dk_csGYfak" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>Next, I tried to test the motor shield and the DC motors with a basic <i>moving forward</i> sketch and the +3.7V 2000mA LiPo battery. The DC motors (3-12V) did not work, and I started debugging step by step. The pin numbers were carefully double-checked, and the sketch was very simple, so I was pretty sure the problem was with the electronic parts:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * libraries and pin variables
     */ 
    int motor1Pin1 = 4; 
    int motor1Pin2 = 16; 

    void setup() {
        pinMode(13, OUTPUT);
        pinMode(motor1Pin1, OUTPUT);
        pinMode(motor1Pin2, OUTPUT);
        Serial.begin(115200);
    }

    void loop() {
        digitalWrite(13, HIGH);   
        delay(500);                     
        digitalWrite(13, LOW);  
        delay(500);
        /* 
         * moving forward
         */ 
        digitalWrite(motor1Pin1, LOW);
        digitalWrite(motor1Pin2, HIGH); 
        delay(2000);
    }
</code>
</pre>
<p>The LED could blink on and off so I knew that the power was transferred safely from the battery to the motor shield and then the ESP32 board. With the help from Oscar and Josep, we supplied a 5V power to the shield and used the multimeter to thoroughly test these following points:</p>
<ul>
<li>When we connected the motors and the battery directly, the motors rotated. Hence, we could confirm that the battery and the motors work fine.</li>
<li>We checked the VREF and voltage levels from the ESP32 micro-controller to the input pins of the A4953 H-Bridge chip. The pin which was written <i>LOW</i> showed 0V and the pin which was written <i>HIGH</i> showed ~3.3V. So these parts were fine.</li>
<li>Next, we checked the voltage levels from the output pins of the H-Bridge chip to the terminal blocks. Both of them showed ~4.8V. After checking again from the code to the schema of the motor shield, then the datasheet of the A4953 H-Bridge chip, we still couldn't figure out why the A4953 chip didn't do what it should do. I even re-soldered and replaced the chips with new ones, but the motors were still motionless.</li>
</ul>
<p>After reading all topics and articles related to the A4953 chip, I found <a href="http://fab.cba.mit.edu/classes/863.13/people/Hasier/Week12.html">this documentation</a> from a CBA student. There are magic lines from the beginning of the documentation which states that the A4953 chip is an automotive-grade chip that has under-voltage protection at 7.9V. That means if it detects input voltage less than 7.9V, it will not output anything. We then supplied a 9V power to the motor shield, and finally, the motors turned alive!</p>
<div class="text-center">
<iframe width="640" height="360" src="https://www.youtube.com/embed/x5J0ozRqNoE" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>

<h2>Conclusion</h2>
<p>It was really amazing to be able to finish the most crucial electronic parts of my Final Project. It might not be too ambitious, but I learned a lot from working on it step by step. I can't wait to finish the proper networking and programming parts, stay tuned!</p>

<div class="container w-100 text-center py-4">
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-10-applications-and-implications.html">Previous Assignment</a>
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-12-interface-and-application-programming.html">Next Assignment</a>
</div>