---
layout: assignment
title: Networking And Communications
thumb: 15.png
assignment: design, build, and connect wired or wireless node(s) with network or bus addresses (individual); to send a message between two projects (in-group). 
active: 1
---

<p class="font-italic font-weight-bold">* This week's <a href="#group">group assignment</a> is to send data between projects. We classmates did it together during an online session regarding MQTT.</p>

<p class="font-italic font-weight-bold">* I also worked on my own <a href="#mqtt">MQTT</a> project to send sensors' data from my ESP32 board to a self-made Node-RED dashboard. Then through Node-RED functions and/or UI, I controlled the actuators of the same board.</p>

<p class="font-italic font-weight-bold">* All source files can be downloaded <a href="https://gitlab.fabcloud.org/academany/fabacademy/2020/labs/barcelona/students/tue-ngo/tree/master/assets/img/network">here</a>.</p>

<h2>Connect wireless nodes</h2>
<h5 id="group">Group assignment - MQTT + Node-RED</h5>
<p>This week, our instructor Oscar set up an MQTT broker on Raspberry Pi and got all the students to send some data from their boards. My classmate <a href="http://fabacademy.org/2020/labs/barcelona/students/david-prieto/assignments/week14/">David Prieto</a> documented very well the whole process, and I also wanted to work on MQTT as my individual assignment this week, so I only put key recap points of the group work here:</p>
<ul>
<li>We subscribed to certain topics to be able to receive messages from others (e.g. <code>client.subscribe("onput/tue", msg);</code>) and we sent messages to others by publishing a message to a certain topics (e.g. <code>client.publish("input/tue", msg);</code>).</li>
<li>Oscar already provided us with a sample program, and all what we had to do was to fill in our own WiFi credentials, to check the pins we were using, and to change the name of the topics we were subscribing or publishing to.</li>
</ul>
<p>And here you go the screenshot with happy faces, captured by David:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/group.jpg" class="img-fluid w-100"/>
<p></p>

<h5 id="mqtt">Individual assignment - MQTT + Node-RED</h5>
<p>From what I understood, it was not mandatory for us to fabricate multiple boards for this week's assignment. The purpose of this assignment was to connect wired or wireless nodes, hence, my plan was to utilize my board for the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/final-project.html">Final Project</a> and to simply send messages from my sensors to my actuators of the same board over a networking protocol. In my case, I decided to try MQTT and added another node to the networking flow, which is a Node-RED dashboard. And thus, I could display sensors' data on the dashboard and control the actuators through Node-RED functions. Somehow somewhat it makes sense, isn't it?</p>
<h6>Mosquitto</h6>
<p>MQTT is a machine-to-machine connectivity protocol. It was designed as an extremely lightweight publish/subscribe messaging transport. An MQTT broker is a server that receives all messages from the clients, and then routes the messages to the appropriate destination clients. The broker is responsible for receiving all messages, filtering the messages, determining who subscribed to each message and sending the message to those subscribed clients. <a href="https://mosquitto.org/">Mosquitto</a> is a widely-used MQTT broker. Usually, the MQTT broker will be installed on a Raspberry Pi for security reasons, but I installed it directly to my Macbook because it was more convenient. I didn't want to buy a Raspberry Pi.</p>
<p>I followed <a href="https://subscription.packtpub.com/book/application_development/9781787287815/1/ch01lvl1sec12/installing-a-mosquitto-broker-on-macos">this instruction</a> to install Mosquitto. Here is the screenshot of Mosquitto installed and connected to Node-RED. Also, after plugging in my ESP32 board, I could detect it as a new client:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/mqtt-2.png" class="img-fluid w-100"/>
<p>In another Terminal window, I installed Node-RED locally following <a href="https://nodered.org/docs/getting-started/local">this instruction</a>. I could see Node-RED installed and connected to my MQTT broker:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/node-red-0.png" class="img-fluid w-100"/>
<p>The installation steps seemed good. Let's move on to create a Node-RED dashboard.</p>
<p></p>

<h6>Node-RED</h6>
<p><a href="https://nodered.org/">Node-RED</a> has usually used for wiring together hardware devices, APIs, and online services over MQTT protocol, but it is not limited to that purpose. It is a powerful tool for building applications with a browser-based editor that makes it easy to wire together flows using the wide range of nodes and nice UI elements that can be deployed to its runtime in a single click.</p>
<p>First, I had to create MQTT nodes for all my inputs and outputs:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/node-red-2.png" class="img-fluid w-100"/>
<p>Then, I needed to create some UI dashboard nodes to display my sensors' data as well as sending payloads from the UI switches:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/node-red-3.png" class="img-fluid w-100"/>
<p>I also created some function nodes, mostly to process the data received from inputs and then to send the related payloads to control the outputs:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/node-red-4.png" class="img-fluid w-100"/>
<p>Here you go the final Node-RED flow, ready to be deployed:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/node-red-5.png" class="img-fluid w-100"/>
<p>And the dashboard was also ready:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/node-red-6.png" class="img-fluid w-100"/>
<p></p>

<h6>ESP32: PubSubClient</h6>
<p>Next, I wrote a program to publish all the sensors' data to Node-RED over MQTT, and also subscribed to the payloads sent by Node-RED. Below is the code I flashed to my ESP32 board:</p>
<pre class="bg-light py-2 mt-0" style="overflow: auto; max-height: 350px;">
<code>
    /* 
     * libraries
     */
    #include &lt;DHT.h&gt;
    #include &lt;NewPing.h&gt;
    #include &lt;Adafruit_NeoPixel.h&gt;
    #include &lt;WiFi.h&gt;
    #include &lt;PubSubClient.h&gt;
    /* 
     * pin variables
     */
    #define TRIGGER_PIN 32
    #define ECHO_PIN 35
    #define DHT_PIN 33
    #define LDR_PIN 34
    #define BUZZ_PIN 14
    #define RGB_PIN 26
    /* 
     * useful variables and initializations
     */
    int ldrValue;
    float tempValue;
    float humValue;
    float distance;
    #define DHTTYPE DHT11
    NewPing sonar(TRIGGER_PIN, ECHO_PIN, 200); 
    DHT dht(DHT_PIN, DHTTYPE);
    #define LED_COUNT 10
    Adafruit_NeoPixel strip(LED_COUNT, RGB_PIN, NEO_GRB + NEO_KHZ800);
    /* 
     * timer variables
     */
    unsigned long now = millis();
    unsigned long lastMeasure = 0;
    /* 
     * wifi & mqtt credentials
     */
    const char* ssid = "SSID";
    const char* password = "PASSWORD";
    const char* mqtt_server = "SERVERADDRESS";
    WiFiClient espClient;
    PubSubClient client(espClient);
    /* 
     * setup connections
     */
    void setup() {
        Serial.begin(115200);
        /* 
         * sensors and actuators setup
         */
        pinMode(BUZZ_PIN, OUTPUT);
        dht.begin();
        strip.begin();           
        strip.show();             
        strip.setBrightness(150); 
        /* 
         * wifi & mqtt setup
         */
        Serial.println();
        Serial.print("Connecting to ");
        Serial.println(ssid);
        WiFi.begin(ssid, pass);
        WiFi.mode(WIFI_STA); 
        while (WiFi.status() != WL_CONNECTED) {
            Serial.print(".");
            delay(500);
        }
        Serial.println("");
        Serial.println("WiFi connected");
        Serial.println("IP address: ");
        Serial.println(WiFi.localIP());
        client.setServer(mqtt_server, 1883);
        client.setCallback(callback);
    }
    /* 
     * loop()
     */
    void loop() {
        /* 
         * refresh mqtt subscriptions
         */
        if (!client.connected()) {
            reconnect();
        }
        client.loop();
        /* 
         * publish every 30 seconds
         */
        now = millis();
        if (now - lastMeasure > 30000) {
            lastMeasure = now;
            // publish sensors' data to MQTT
            char tempMsg[50];
            snprintf (tempMsg, 50, "%f", readTemp());
            char humMsg[50];
            snprintf (humMsg, 50, "%f", readHum());
            char luxMsg[50];
            snprintf (luxMsg, 50, "%f", readLDR())
            client.publish("esp32/temperature", tempMsg); 
            client.publish("esp32/humidity", humMsg);
            client.publish("esp32/light", luxMsg);
            if (readDistance() <= 8) {
                client.publish("esp32/sonar", "OBJECT DETECTED!"); 
            } else {
                client.publish("esp32/sonar", "NO OBJECT DETECTED!"); 
            }
        }
    }
    /* 
     * callback function
     */
    void callback(char* topic, byte* payload, unsigned int length) {
        Serial.print("Message arrived on topic: ");
        Serial.print(topic);
        Serial.print(". Message: ");
        String messageTemp;
        // check payload from MQTT
        for (int i = 0; i < length; i++) {
            Serial.print((char)payload[i]);
            messageTemp += (char)payload[i];
        }
        Serial.println();
        // turn LED and buzzer on
        if (topic == "esp32/led") { 
            if (messageTemp == "1") {
            blinkWhite();
            }
        }
        if (topic == "esp32/buzzer") { 
            if (messageTemp == "1") {
            buzz();
            }
        }
        Serial.println(); 
    }
    /* 
     * subscribe to topics
     */
    void reconnect() {
        while (!client.connected()) {
            Serial.print("Attempting MQTT connection...");
            String clientId = "ESP32Client-";
            clientId += String(random(0xffff), HEX);
            // subscribe
            if (client.connect(clientId.c_str())) {
                Serial.println("connected");
                client.subscribe("esp32/led");
                client.subscribe("esp32/buzzer");
            } else {
                Serial.print("failed, rc=");
                Serial.print(client.state());
                Serial.println(" try again in 5 seconds");
                delay(5000);
            }
        }
    }
    /* 
     * read sensors
     */
    int readLDR() {
        delay(500);  
        ldrValue = analogRead(LDR_PIN);
        return ldrValue;
    }
    int readTemp() {
        delay(500);  
        tempValue = dht.readTemperature();
        return tempValue;  
    }
    int readHum() {
        delay(500);  
        humValue = dht.readHumidity();  
        return humValue;
    }
    int readDistance() {
        delay(500);
        distance = sonar.ping_cm();
        return distance; 
    }
    /* 
     * control actuators
     */
    void buzz() {
        digitalWrite(BUZZ_PIN, HIGH);   
        delay(500);                       
        digitalWrite(BUZZ_PIN, LOW);    
        delay(500);
    }
    void blinkWhite() {
        colorWipe(strip.Color(255, 255, 255), 500);
    }
    void colorWipe(uint32_t color, int wait) {
        for(int i = 0; i < strip.numPixels(); i++) { 
            strip.setPixelColor(i, color);         
            strip.show();                          
            delay(wait);                          
        }
    }
}
</code>
</pre>
<p>I could see the board was connected to WiFi, however, I could not establish an MQTT connection.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/pubsub-0.png" class="img-fluid w-100"/>
<p>After debugging for a while, I realized that I was using the IP address of my WiFi router as the server address instead of the the IP address of the Wifi hotspot provided by my phone (of which I used as WiFi credentials in my Arduino sketch). After using the correct IP address, I could successfully send messages over MQTT to control the buzzer of my board:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/pubsub-1.png" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/pubsub-2.png" class="img-fluid w-100"/>
<p>And I could also see updates on the Node-RED dashboard. Yay!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-14/pubsub-3.png" class="img-fluid w-100"/>
<p></p>

<h2>Conclusion</h2>
<p>I was looking for ways to enhance the communication part of my Final Project. MQTT and Node-RED seem to be elegant and promising solutions. I still need to think further about how to get the most out of MQTT and deploy useful features for my Final Project.</p>
<p></p>

<!--
<div class="sketchfab-embed-wrapper text-center">
    <iframe title="A 3D model" width="640" height="480" src="https://sketchfab.com/models/21d3805960c34c5bb9956b13d56ba88d/embed?autostart=1&amp;ui_controls=1&amp;ui_infos=1&amp;ui_inspector=1&amp;ui_stop=1&amp;ui_watermark=1&amp;ui_watermark_link=1" frameborder="0" allow="autoplay; fullscreen; vr" mozallowfullscreen="true" webkitallowfullscreen="true"></iframe>
</div>
<p></p>
<div class="sketchfab-embed-wrapper text-center">
    <iframe title="A 3D model" width="640" height="480" src="https://sketchfab.com/models/e46d51e6d2dc488f8fcc3048c050bdd2/embed?autostart=1&amp;ui_controls=1&amp;ui_infos=1&amp;ui_inspector=1&amp;ui_stop=1&amp;ui_watermark=1&amp;ui_watermark_link=1" frameborder="0" allow="autoplay; fullscreen; vr" mozallowfullscreen="true" webkitallowfullscreen="true"></iframe>
</div>
<p></p>
-->

<div class="container w-100 text-center py-4">
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-13-invention-ip-and-income.html">Previous Assignment</a>
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-15-molding-and-casting.html">Next Assignment</a>
</div>

<!--
<p>Communications protocols are standardized and diversified into the Internet protocol suite which is a conceptual model used in the Internet and similar computer networks:</p>

Finally, OSI layer is all about the interfaces and protocol for the different layers, this is a good source of information and basic understanding of communication system and architecture.
Protocols are rules. They define exactly how data is exchanged and the expected behavior. These are rules you must follow exactly or your device will not be able to communicate at all with other devices. You can not exceed the protocol but its possible that you do not implement all the features of the protocols or you may implement the protocol completely. A very common example is of HTTP. Not every server/browser implements all the features of HTTP for example some may not implement Range transfers in GET/POST but they are still able to work with browsers. You will notice sometimes you are able to continue file download where you left in case it was aborted whereas in some cases you have to restart the file download from scratch even if you have previously downloaded 99% of the file. Its because the server did not implement the RANGE feature of HTTP protocol.

Higher layers build on top of the other one, and less work to do.


<ul>
<li>The link layer is responsible for node-to-node transmission and reception of unstructured raw data between a device and a physical transmission medium. Layer specifications define characteristics such as voltage levels, signal timing, modulation scheme, specifications of physical connectors, and frequency for wireless devices. Examples: Ethernet, Wifi, Bluetooth, CAN, USB, etc.</li>
<li>The Internet layer provides the functional and procedural means of transferring variable length data sequences (called packets) from the originating host to the destination host specified by an IP address, with a network as a medium. Examples: IP, DDP, etc.</li>
<li>The transport layer establishes host-to-host connectivity and data channels that applications use for task-specific data (also called data segments) exchange. The best-known transport protocol is the Transmission Control Protocol (TCP). It is used for connection-oriented transmissions, whereas the connectionless User Datagram Protocol (UDP) is used for simpler messaging transmissions.</li>
<li>The application layer is the closest layer to end users, and allow users to interact directly with the application. The application layer only standardizes communication and depends upon the underlying transport layer protocols to establish host-to-host data transfer channels and manage the data exchange in a client-server or peer-to-peer networking model. Examples: HTTP/HTTPS, SSH, MQTT, etc.</li>
</ul>
https://www.esp8266.com/viewtopic.php?p=75612
https://nodered.org/docs/user-guide/writing-functions
https://subscription.packtpub.com/book/application_development/9781787287815/1/ch01lvl1sec12/installing-a-mosquitto-broker-on-macos
https://circuits4you.com/2018/12/31/esp32-pwm-example/
-->