---
layout: assignment
title: Wildcard Week
thumb: 17.png
assignment: design and produce something with a digital fabrication process not covered in other assignments. 
active: 1
---

<p class="font-italic font-weight-bold">* This week's assignment is to work with a digital fabrication process <a href="#clay">not covered in other assignments.</a></p>

<p class="font-italic font-weight-bold">* All readable and original editable files can be downloaded <a href="https://gitlab.fabcloud.org/academany/fabacademy/2020/labs/barcelona/students/tue-ngo/tree/master/assets/img/wildcard">here</a>.</p>

<h2>Paste extrusion printing</h2>
<h5 id="clay">Individual assignment - Food printing/Clay printing using a syringe</h5>
<p>Paste extrusion works by dispensing any paste material through a user-filled syringe, and we can print edible materials like icing sugar, Nutella, or polyurethane silicone. In the <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-05-3d-printing-and-scanning.html">5th week</a>, we had Secil Afsar as a guest instructor who gave us an interesting session explaining how she prints pottery-like models with FDM printers, using a variety of materials such as clay or edible materials (I tasted them, pretty good!). Since I was super curious about the multiple tests she did during her process of calibrating the printer, I convinced her to print one of my models, then I could document the process. She also helped me study the Grasshopper file she used to generate <code>.gcode</code> files. Thanks, Secil!</p>
<p></p>
<h6>Designing with Grasshopper</h6>
<p>My biggest problem with the way we're doing paste extrusion printing at Fab Lab Barcelona was that I'm not comfortable with Grasshopper. To be honest, I didn't think any of the models I've had designed with other software was suitable for this technique. Viscosity and curing time are very important aspects to consider when designing for paste extrusion. Thick paste materials are heavy and are very likely to fall down during the process if there are overhangs. They also take hours or even days to be dry and solidified, therefore support structures might not be possible. In the end, I quickly created a very simple star-shaped pattern:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/grass-1.png" class="img-fluid w-100"/>
<p>In order to generate a custom G-CODE, we used a Grasshopper file from Eduardo in which he already prepared the scripts. I added my design as a <strong>Brep</strong> into Eduardo's, and modify some parameters. I tried this technique with 2 FDM printers, but here I only documented the 2nd try with the <a href="https://www.anycubic.com/products/anycubic-kossel-3d-printer">Anycubic Kossel Plus</a> delta printer. I also modified the <strong>Delta Printer Diameter</strong> to be <i>220</i> and <strong>Delta Printer Height</strong> to be <i>270</i>.</p>
<p></p>
<h6>Slicing with Grasshopper</h6>
<p>The slicing process this time was different since the material we used is not required to be melted, therefore we didn't need to deal with factors like material temperature or infill density. Also, because we used a syringe as the extrusion head, we would like to customize the settings, especially the nozzle diameter and printing speed. Speed, air pressure, and nozzle play a very big role in paste extrusion. The syringe can swell under the pressure of a thick mixture. If there is too much pressure, we need to slow the print speed and/or increase the nozzle diameter.</p>

<p>Eduardo guided me through the whole process. First, we needed to find the "fake" Z0 by manually controlling the FDM printer using <a href="https://www.repetier.com/download-now/">Repetier Host</a>. In order to do that, we sent several G-CODE commands with the format <code>G1 F400 Z<i>[some negative number]</i></code> until the nozzle gently touched the bed. After calibrating, we could know it's <strong>Z-23</strong>. Here you go a series of quick calculation:</p>
<ul>
<li>Measure the <strong>Nozzle Diameter</strong>. In this case, the nozzle diameter is <i>1.8</i>.</li>
<li>The <strong>Slicing Distance</strong> is more or less 60-70% of the nozzle diameter, and I put here <i>1.1-1.2</i></li>
<li><strong>Z Height Offset</strong> is an important number, and we should make sure <strong><i>Z Height Offset - Slicing Distance</i></strong> to be a bit less than <strong>Z-23</strong> (we can check the generated G-CODE to know). Hence, I put <strong>Z Height Offset</strong>=<i>23.8</i>.</li>
</ul>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/grass-2.png" class="img-fluid w-100"/>
<p>Then the most important parameters that we should pay attention to while extruding paste:</p>
<ul>
<li><strong>Printing Speed</strong>: we can modify this parameter via Grasshopper or override it in Repetier Host. I put <i>800</i> in my case.</li>
<li>The air pressure will be manually controlled via the air pressure valve system. In Grasshopper, we can modify <strong>Extrusion Wait</strong> which is the waiting time between an extrusion and the next.</li>
</ul>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/grass-3.png" class="img-fluid w-100"/>
<p>The final G-CODE was generated and ready to use. We could <strong>Copy Data Only</strong> and print now!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/grass-4.png" class="img-fluid w-100"/>
<p></p>
<h6>Printing edible paste with Repetier Host</h6>
<p>I tried to print an edible paste similar to peanut butter first. Secil helped me with the process. To run the custom G-CODE, we connected the printer to the computer and pasted the G-CODE into the <strong>G-Code Editor</strong> tab of <a href="https://www.repetier.com/download-now/">Repetier Host</a>:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/repetier-1.png" class="img-fluid w-100"/>
<p>We could also manually control the printing process through the <strong>Manual Control</strong> tab. The <strong>Feedrate</strong> section here means how many percentages of the printing speed (the speed we fed in Grasshopper) that we wanted to use. By turning the <strong>Fan</strong> on/off, we could immediately start and stop pressing the syringe.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/repetier-2.png" class="img-fluid w-100"/>
<p></p>
<p>We used the <a href="https://shop.prusa3d.com/en/51-original-prusa-i3-mk3s">Prusa i3 MK3S</a> printer to print the model (I'm not sure about the correct version of the printer though). The customized extruder consists of a syringe that has been created to fit the original FDM 3D printer. Some basic specs of the machine:</p>
<ul>
<li>Printing technology: FDM</li>
<li>Printing size: 250 x 210 x 200mm</li>
<li>Maximum extruder temperature: 300ºC</li>
<li>Firmware: Marlin</li>
<li>Printing method: SD card, USB</li>
</ul>
<img src="{{site.baseurl}}/assets/img/assignments/week-05/prusa-4.jpg" class="img-fluid w-100"/>
<p>After sending G-CODE to the printer, the extrusion head started to move correctly in accordance with the pre-defined paths, but the syringe didn't pump properly. After investigating for a while, it could be some issues with the motors of the machine. We decided to leave it like that, then Secil could continue calibrating the printer.</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/5wRxCFEQtoM" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<h6>Printing clay with Repetier Host</h6>
<p>When I got back to the lab after the COVID-19 lockdown, Eduardo helped me to resume the process using the <a href="https://www.anycubic.com/products/anycubic-kossel-3d-printer">Anycubic Kossel Plus</a> printer. I also try a different material, which is clay. The good thing about printing with clay is that we can add water to adjust the viscosity to help get you the print we’re after. As it’s a thicker mixture, we could include overhangs and longer bridges while printing at a slower speed.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/clay-1.jpg" class="img-fluid w-100"/>
<p>At Fab Lab Barcelona, we used a system consisting of 3 pressure regulators. We mostly worked with the pressure regulator attached to the 3D printer, and this one directly controls the pressure applied to the syringe.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/clay-2.jpg" class="img-fluid w-100"/>
<p>The next important step was mixing material. I tried to ask for a formula before mixing, but the answer I got from the people at IAAC was "Just feel it!". So...let's give it a try! It took me plenty of time to mix the clay and to reach a good level of viscosity, but there is nothing much to say about the process.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/clay-3.jpg" class="img-fluid w-100"/>
<p>Below is a video recording the printing process:</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/IoeDgzzH5K4" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>Paste extrusion might sound simple but it actually wasn't. If the pressure gets too high, the printing result could be destroyed. If the pressure gets too low, the extrusion process might be interrupted and layers would not be smooth and even. Here I have an example of how a high air pressure destroyed a corner of my star-shaped object:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/clay-4.jpg" class="img-fluid w-100"/>
<p>After the first attempt, I still had much material left. In honor of my fatigue arms, I tried to print something else. I decided to print a cat and quickly drew it with Illustrator:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/cat.png" class="img-fluid w-100"/>
<p>In order to keep the printing and let it dry later, I put a piece of plywood on top of the printing bed. Then I made a huge mistake: I didn't re-calibrate the Z0, and the nozzle hit the bed so hard that it was destroyed. I had to replace the nozzle with a new one, put a piece of paper on top of the printing bed instead, and started printing again.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/clay-5.jpg" class="img-fluid w-100"/>
<p>In this 2nd printing attempt, I reduced the air pressure, and I have another example of how a low pressure made my final layer slightly smudged. Anyway, I was already happy with the final result!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/clay-6.jpg" class="img-fluid w-100"/>
<p>After half a day, my cat was dried. In theory, we should cure the clay by baking it in an oven, so that the final result won't get too brittle or won't be melted under high moisture. Unfortunately, the oven was not in the lab at the moment. I thought I had gone through the most important part of paste extrusion printing, so I stopped the process here. And...ouch! The cat was broken, as expected!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-16/hero.jpg" class="img-fluid w-100"/>
<p></p>

<h2>Conclusion</h2>
<p>I'm personally very fond of paste printing since it allows us to mix hard and flexible materials in a single design all with existing FDM 3D printers. I started trying this technique before the lockdown, and I was quite happy that I could manage to finish it at the end. However, it was a pity that I couldn't try other "wild" techniques mentioned by Neil. 6 months was not enough to learn everything I want to learn!</p>

<div class="container w-100 text-center py-4">
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-15-molding-and-casting.html">Previous Assignment</a>
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-17-mechanical-and-machine-design.html">Next Assignment</a>
</div>
