---
layout: assignment
title: Mechanical and Machine Design
thumb: 18.png
assignment: design, actuate, and automate a machine (in-group). 
active: 1
---
<p class="font-italic font-weight-bold">* This week's assignment is to design, actuate, and automate a machine. I teamed up with <a href="http://fabacademy.org/2020/labs/barcelona/students/david-prieto/assignments/week17/">David Prieto</a> for this assignment.</p>

<p class="font-italic font-weight-bold">* We built our machine as a derivation of the <a href="#spml">multi-tool SPML</a> developed by our instructors at Fab Lab Barcelona. We made a <a href="#light">light-painting</a> machine that makes movements in 3D that can be captured with a camera.</p>

<p class="font-italic font-weight-bold">* The machine group page was made by myself and is updated <a href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/barcelona-machines/light-machine/">here</a>.</p>

<p class="font-italic font-weight-bold">* All readable and original editable files can be downloaded <a href="https://gitlab.fabcloud.org/academany/fabacademy/2020/labs/barcelona/barcelona-machines/light-machine/-/tree/master/assets/files">here</a>.</p>

<h2 id="spml">Understanding SPML and GRBLESP32</h2>
<h5>SPML a.k.a SixPackMobileLab or SimplePersonalMiniLab</h5>
<p>This machine started as a project to make a Fab Academy kit machine to help the students develop their Fab Academy assignments during the COVID-19 confinement. Basically, it is a 3-axes machine with a tool work area of 100 x 100 x 80mm and an exterior footprint of 150 x 150 x 200mm that makes it really portable, lightweight, and extremely low cost. There are 2 options for fabrication: laser-cut or 3D-printed. It uses a modular tool head (laser and CNC) and low current motors that could be driven with 5V input, it has an ESP32 control board with USB-WiFi-Bluetooth control.</p>
<p></p>
<h5>Barduino 2.0 and GBRLESP32 firmware</h5>
<p>The electronic parts of the SPML include a <a href="https://gitlab.fabcloud.org/barcelonaworkshops/barduino-2.0">Barduino 2.0</a> control board and its <a href="https://gitlab.com/fablabbcn-projects/electronics/barduino-cnc-shield">CNC shield</a> to control the actuators. The machine uses <a href="https://github.com/bdring/Grbl_Esp32/wiki/Development-Roadmap">GBRLESP32</a> firmware with <a href="https://github.com/luc-github/ESP3D-WEBUI">a self-publish ESP3D-WebUI</a> to control the machine through the browser via WiFi or Bluetooth.</p>
<p></p>

<h2 id="light">Light-painting machine</h2>
<p>Our idea was to build a 3D light-painting machine that can move and paint with an LED so that later the whole movement can be captured with a camera. We planned to leverage the <a href="https://gitlab.fabcloud.org/barcelonaworkshops/barduino-2.0">Barduino 2.0</a> board and its <a href="https://gitlab.com/fablabbcn-projects/electronics/barduino-cnc-shield">CNC shield</a>, with some customizations.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/sketch-0.jpeg" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/sketch.jpg" class="img-fluid w-100"/>
<p></p>

<h5>Mounting the machine</h5>
<h6>Designing 3D mounts</h6>
<p>David completely handled this task so he documented this part better in <a href="http://fabacademy.org/2020/labs/barcelona/students/david-prieto/assignments/week17/">his documentation</a>. I only put some pictures of his design here:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/design-1.jpg" class="img-fluid w-100"/>
<p>David took away all the laser-cut frames from the original SPML since moving a LED strip/panel doesn't require something as sturdy as if there is a spindle/tool. He kept the bottom platform because it had the X-home switch holder. He re-designed all adapters for the Y-home switch, the Z-axis holder, and the LED panel/strip holder.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/design-2.jpg" class="img-fluid w-100"/>
<p></p>

<h6>Printing 3D mounts</h6>
<p>This was also handled by David, with the supports of our instructors Mikel and the interns at Fab Lab Barcelona:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/3dprint-0.jpg" class="img-fluid w-100"/>
<p>I helped a bit in screwing the mounts to the machine. This was not an easy and comfortable task. The holes were too small so the screws had to be screwed twice to fit in and there were some tolerances in both measuring the distances and printing. Also, the X and Y connections were very difficult to screw because there is very little space for us to freely work on them.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/3dprint-1.jpg" class="img-fluid w-100"/>
<p></p>

<h5>Making the connections</h5>
<p>Now the most interesting part, with a lot of help from our classmate Arman. Basically, stepper motors are DC motors that move in discrete steps. They have multiple coils that are organized in groups called "phases". By energizing each phase in sequence, the motor will rotate, one step at a time. The number of steps per revolution ranges from 4 to 400. Commonly available step counts are 24, 48 and 200. Resolution is often expressed as degrees per step. A 1.8° motor is the same as a 200 step/revolution motor. Here we had 3 unipolar 6-wire motors for 3 axes, and 3 bipolar A4988 bipolar drivers, each with 4 pins. In order to solve that, we came up with a schema to understand how we should make the connections as below:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/stepper-0.jpeg" class="img-fluid w-100"/>
<p>Speaking of the unipolar motors, they always energize the phases in the same way: the "common" lead will always be negative, the other lead will always be positive. The 6-wire motors only join the common wires of 2-paired phases. These 2 wires can be joined to create a 5-wire unipolar motor or we just can ignore them and treat the motors as bipolar ones. What we did was to go with the latter option and to measure the resistance between wires as shown in the schema. The wires that we connected were those that had 60Ω resistance between them.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/stepper-1.jpg" class="img-fluid w-100"/>
<p></p>

<h5>Working with GBRLESP32</h5>
<h6>Flashing GBRLESP32 firmware to the Barduino board</h6>
<p>First, we had to download the whole firmware including all configuration files and the web interface <a href="https://gitlab.com/fablabbcn-projects/cnc-machines/six-pack-cnc/-/tree/master/firmware-grblesp32/firmware-3AxisBrushedMotor/Grbl_Esp32">here</a>. We also needed to install all the necessary <a href="https://gitlab.com/fablabbcn-projects/cnc-machines/six-pack-cnc/-/tree/master/firmware-grblesp32/libraries">libraries</a> and the <code>index.html.gz</code> file <a href="https://gitlab.com/fablabbcn-projects/cnc-machines/six-pack-cnc/-/tree/master/firmware-grblesp32/firmware-3AxisBrushedMotor/Grbl_Esp32/data">here</a>. It took us a long while but we failed to flash the firmware to the ESP32 chip. The chip kept registering forever and ever. Later we figured out that we should choose the proper <strong>Flash Size</strong>:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/flash-2.png" class="img-fluid w-100"/>
<p>We also had to choose the <i>Minimal SPIFFS</i> option in the <strong>Partition Scheme</strong> section:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/flash-1.png" class="img-fluid w-100"/>
<p>Finally, we could find a new visible WiFi in our PC with ESP_GRBL name, which means we had flashed our ESP correctly. In the case of David, he had to connect to that network and navigate to <code>http://192.168.4.1</code> in the browser. In my case (I'm using Macbook), I only needed to connect to that WiFi and a window popped out immediately.</p>
<p></p>

<h6>Sending commands and GRBL configurations</h6>
<p>In the web interface, we have 3 tabs. The most important one is the dashboard for sending commands, directly control the 3 axes, modifying feed and speed, and switch on/off some outputs (spindle, mist, or flood).</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/grbl-1.png" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/grbl-2.png" class="img-fluid w-100"/>
<p></p>

<h5>Controlling stepper motors</h5>
<p>After being able to access the GBRLESP32 web interface, we tested the stepper motors. We had a problem right away that the motors were trying to rotate but got stuck:</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/mSGB3OjmaDw" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>
<p>To address this, we discussed with instructors Oscar and Eduardo and they suggested us to try these debugging steps:</p>
<ul>
<li>Oscar suggested us to check the regulated current of the stepper driver. We used the <a href="https://www.pololu.com/product/1182">Polulu A4988</a> micro-stepping bipolar stepper motor driver that features adjustable current limiting, over-current and over-temperature protection. The maximum rated current that we should set the stepper driver to output is around 1.4 - 1.68A. This is important in order to provide the ideal amount of torque for the extruder to run without skipping steps or overheating and damaging the motor. In order to adjust the output current, we used a screwdriver.</li>
<li>Since we could not measure current in parallel, Eduardo suggested us to check <a href="https://e3d-online.dozuki.com/Guide/VREF+adjustment+A4988/92">this reference</a> in order to adjust the VREF instead. The formula used is <i><strong>VREF = maximum rated motor current x 8 x Rsense</strong></i>. In order to get an accurate result from the calculation we need to work out what resistors our A4988 used, and that number is <i>Rsense=0.05Ω</i>. So, the final VREF we wanted to adjust to was <strong>~0.6A</strong>.</li>
</ul>
<p>After adjusting the VREF (and thus, the regulated current), only the Z-axis was able to rotate and move perfectly, we still had problems with the X-axis and the Y-axis. We also tried to use our board and CNC shield to control the machine from our classmate <a href="http://fabacademy.org/2020/labs/barcelona/students/arman-najari/assignments/week17/">Arman Narjari</a>, and it worked. But it was not the case for our own machine. In the end, we decided to substitute the stepper drivers, and David re-designed the 3D-printed mounts as well.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/driver-1.jpg" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/3dprint-new.jpg" class="img-fluid w-100"/>
<p>Yay! Finally! They moved as we wanted!</p>
<div class="text-center">
<iframe width="640" height="480" src="https://www.youtube.com/embed/Dw5LUpdOSkg" frameborder="0" allow="accelerometer; autoplay; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
</div>
<p></p>

<h5>Controlling the LED</h5>
<p>At first, we wanted to include an addressable LED strip for a fancier display of the light, and an IR receiver to receive signals from the camera. Here is the board I designed for those purposes, with pin headers connecting to the LED strip and the available input/output pins of the CNC shield (<i>Flood</i>, <i>Mist</i>, or <i>Door</i>):</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/ir-1.png" class="img-fluid w-100"/>
<p>However, we were running out of time and we didn't want to work further on digesting how to send the correct data signal to the addressable LED strip. Therefore, we changed to a normal LED attached to the <i>Spindle_PWM_3V3</i> pin and re-soldered the connections. So basically, we could simply turn on/off the LED with a certain brightness by sending <code>M3</code>/<code>M5</code> commands with a certain spindle speed (a lot of assumptions were made here!!!). And fortunately, it worked!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/final-0.jpg" class="img-fluid w-100"/>
<p>As per <a href="https://smithy.com/cnc-reference-info/chapter-5/spindle-control/page/1">this reference</a>, <code>M3</code> is used to start the spindle turning clockwise, <code>M4</code> is used to start the spindle turning counterclockwise, and <code>M5</code> is used to stop the spindle from turning. In our case, <code>M3</code> is used to turn on the LED, and <code>M5</code> is used to turn off the LED.</p>
<p></p>

<h5>Final machine</h5>
<p>Here you go some hero shots of the final machine. It looks a bit like a Frankenstein-style machine, but it works!</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/final-1.jpg" class="img-fluid w-100"/>
<p></p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/final-2.jpg" class="img-fluid w-100"/>
<p></p>

<h5>Testing the machine</h5>
<h6>Generating G-CODE with Cura</h6>
<p>In order to test the machine, first, we made a specific fake 3D printer resembling our light-painting machine with some below settings:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/cura-1.jpeg" class="img-fluid w-100"/>
<p>The <code>M5 G0 XO YO ZO</code> end G-CODE means stopping the LED and go to X,Y,Z home. Then we sliced the model and saved the <code>.gcode</code> file. The next step was to modify that <code>.gcode</code> file to make sure of its compatibility with GBRLESP32 commands and our machine. In summary, we had to make these modifications:</p>
<ul>
<li>Delete all the Fan-Off/Laser-Off commands: <code>M107</code></li>
<li>Comment out all <code>Ex.xx</code> commands by adding a semicolon before: <code>;Ex.xx</code></li>
<li>Add LED-On commands before every <code>G1</code> commands: <code>M3 S4000</code></li>
<li>Add LED-Off commands before every <code>G0</code> commands: <code>M5</code></li>
</ul>
<p>We repeated the above-mentioned process with 2 models: a joint from David's Final Project and a <a href="https://www.thingiverse.com/thing:1222017/files">puppy</a>:</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/slicer-1.jpg" class="img-fluid w-100"/>
<h6>Capturing light movements</h6>
<p>David documented well his process of taking the pictures <a href="http://fabacademy.org/2020/labs/barcelona/students/david-prieto/assignments/week17/">here</a>, but I'd like to recap briefly here. We found a little space among the shelves in the lab with a completely black background and put dark cloths to make it even darker. Also, David's camera (Canon EOS 450D) has 2 options for long exposure, either up to 30 seconds or as long as the button is pressed. Hence, instead of a one-take, David took multiple 30s shots and then combined them in Photoshop.</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/hero-1.jpg" class="img-fluid w-100"/>
<p>Wonderful, isn't it?</p>
<img src="{{site.baseurl}}/assets/img/assignments/week-17/hero-2.jpg" class="img-fluid w-100"/>
<p></p>

<h2>Conclusion</h2>
<p>At first, I was a bit confused with this assignment, since all of the concepts and practices of motion-control firmware, CNC shields, and stepper motors took me a lot of time digesting. This final machine also required a high extent of integration and complexity, as compared to that of the Final Project. However, I was quite fortunate to team up with David. He has good project management skills and a great ability to improvise. I felt very happy that I've learned a lot after this assignment.</p>
<p></p>

<div class="container w-100 text-center py-4">
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-16-wildcard-week.html">Previous Assignment</a>
<a class="btn m-2" href="http://academany.fabcloud.io/fabacademy/2020/labs/barcelona/students/tue-ngo/assignments/week-18-project-development.html">Next Assignment</a>
</div>